// Generated by swizzlegen. Do not edit.
#[macro_use]
mod support;
use glam::*;

glam_test!(test_isizevec4_swizzles, {
    let v = isizevec4(1_isize, 2_isize, 3_isize, 4_isize);
    let rhs3 = isizevec3(11_isize, 12_isize, 13_isize);
    let rhs2 = isizevec2(11_isize, 12_isize);
    assert_eq!(v, v.xyzw());
    assert_eq!(v.xxxx(), isizevec4(1_isize, 1_isize, 1_isize, 1_isize));
    assert_eq!(v.xxxy(), isizevec4(1_isize, 1_isize, 1_isize, 2_isize));
    assert_eq!(v.xxxz(), isizevec4(1_isize, 1_isize, 1_isize, 3_isize));
    assert_eq!(v.xxxw(), isizevec4(1_isize, 1_isize, 1_isize, 4_isize));
    assert_eq!(v.xxyx(), isizevec4(1_isize, 1_isize, 2_isize, 1_isize));
    assert_eq!(v.xxyy(), isizevec4(1_isize, 1_isize, 2_isize, 2_isize));
    assert_eq!(v.xxyz(), isizevec4(1_isize, 1_isize, 2_isize, 3_isize));
    assert_eq!(v.xxyw(), isizevec4(1_isize, 1_isize, 2_isize, 4_isize));
    assert_eq!(v.xxzx(), isizevec4(1_isize, 1_isize, 3_isize, 1_isize));
    assert_eq!(v.xxzy(), isizevec4(1_isize, 1_isize, 3_isize, 2_isize));
    assert_eq!(v.xxzz(), isizevec4(1_isize, 1_isize, 3_isize, 3_isize));
    assert_eq!(v.xxzw(), isizevec4(1_isize, 1_isize, 3_isize, 4_isize));
    assert_eq!(v.xxwx(), isizevec4(1_isize, 1_isize, 4_isize, 1_isize));
    assert_eq!(v.xxwy(), isizevec4(1_isize, 1_isize, 4_isize, 2_isize));
    assert_eq!(v.xxwz(), isizevec4(1_isize, 1_isize, 4_isize, 3_isize));
    assert_eq!(v.xxww(), isizevec4(1_isize, 1_isize, 4_isize, 4_isize));
    assert_eq!(v.xyxx(), isizevec4(1_isize, 2_isize, 1_isize, 1_isize));
    assert_eq!(v.xyxy(), isizevec4(1_isize, 2_isize, 1_isize, 2_isize));
    assert_eq!(v.xyxz(), isizevec4(1_isize, 2_isize, 1_isize, 3_isize));
    assert_eq!(v.xyxw(), isizevec4(1_isize, 2_isize, 1_isize, 4_isize));
    assert_eq!(v.xyyx(), isizevec4(1_isize, 2_isize, 2_isize, 1_isize));
    assert_eq!(v.xyyy(), isizevec4(1_isize, 2_isize, 2_isize, 2_isize));
    assert_eq!(v.xyyz(), isizevec4(1_isize, 2_isize, 2_isize, 3_isize));
    assert_eq!(v.xyyw(), isizevec4(1_isize, 2_isize, 2_isize, 4_isize));
    assert_eq!(v.xyzx(), isizevec4(1_isize, 2_isize, 3_isize, 1_isize));
    assert_eq!(v.xyzy(), isizevec4(1_isize, 2_isize, 3_isize, 2_isize));
    assert_eq!(v.xyzz(), isizevec4(1_isize, 2_isize, 3_isize, 3_isize));
    assert_eq!(v.xywx(), isizevec4(1_isize, 2_isize, 4_isize, 1_isize));
    assert_eq!(v.xywy(), isizevec4(1_isize, 2_isize, 4_isize, 2_isize));
    assert_eq!(v.xywz(), isizevec4(1_isize, 2_isize, 4_isize, 3_isize));
    assert_eq!(v.xyww(), isizevec4(1_isize, 2_isize, 4_isize, 4_isize));
    assert_eq!(v.xzxx(), isizevec4(1_isize, 3_isize, 1_isize, 1_isize));
    assert_eq!(v.xzxy(), isizevec4(1_isize, 3_isize, 1_isize, 2_isize));
    assert_eq!(v.xzxz(), isizevec4(1_isize, 3_isize, 1_isize, 3_isize));
    assert_eq!(v.xzxw(), isizevec4(1_isize, 3_isize, 1_isize, 4_isize));
    assert_eq!(v.xzyx(), isizevec4(1_isize, 3_isize, 2_isize, 1_isize));
    assert_eq!(v.xzyy(), isizevec4(1_isize, 3_isize, 2_isize, 2_isize));
    assert_eq!(v.xzyz(), isizevec4(1_isize, 3_isize, 2_isize, 3_isize));
    assert_eq!(v.xzyw(), isizevec4(1_isize, 3_isize, 2_isize, 4_isize));
    assert_eq!(v.xzzx(), isizevec4(1_isize, 3_isize, 3_isize, 1_isize));
    assert_eq!(v.xzzy(), isizevec4(1_isize, 3_isize, 3_isize, 2_isize));
    assert_eq!(v.xzzz(), isizevec4(1_isize, 3_isize, 3_isize, 3_isize));
    assert_eq!(v.xzzw(), isizevec4(1_isize, 3_isize, 3_isize, 4_isize));
    assert_eq!(v.xzwx(), isizevec4(1_isize, 3_isize, 4_isize, 1_isize));
    assert_eq!(v.xzwy(), isizevec4(1_isize, 3_isize, 4_isize, 2_isize));
    assert_eq!(v.xzwz(), isizevec4(1_isize, 3_isize, 4_isize, 3_isize));
    assert_eq!(v.xzww(), isizevec4(1_isize, 3_isize, 4_isize, 4_isize));
    assert_eq!(v.xwxx(), isizevec4(1_isize, 4_isize, 1_isize, 1_isize));
    assert_eq!(v.xwxy(), isizevec4(1_isize, 4_isize, 1_isize, 2_isize));
    assert_eq!(v.xwxz(), isizevec4(1_isize, 4_isize, 1_isize, 3_isize));
    assert_eq!(v.xwxw(), isizevec4(1_isize, 4_isize, 1_isize, 4_isize));
    assert_eq!(v.xwyx(), isizevec4(1_isize, 4_isize, 2_isize, 1_isize));
    assert_eq!(v.xwyy(), isizevec4(1_isize, 4_isize, 2_isize, 2_isize));
    assert_eq!(v.xwyz(), isizevec4(1_isize, 4_isize, 2_isize, 3_isize));
    assert_eq!(v.xwyw(), isizevec4(1_isize, 4_isize, 2_isize, 4_isize));
    assert_eq!(v.xwzx(), isizevec4(1_isize, 4_isize, 3_isize, 1_isize));
    assert_eq!(v.xwzy(), isizevec4(1_isize, 4_isize, 3_isize, 2_isize));
    assert_eq!(v.xwzz(), isizevec4(1_isize, 4_isize, 3_isize, 3_isize));
    assert_eq!(v.xwzw(), isizevec4(1_isize, 4_isize, 3_isize, 4_isize));
    assert_eq!(v.xwwx(), isizevec4(1_isize, 4_isize, 4_isize, 1_isize));
    assert_eq!(v.xwwy(), isizevec4(1_isize, 4_isize, 4_isize, 2_isize));
    assert_eq!(v.xwwz(), isizevec4(1_isize, 4_isize, 4_isize, 3_isize));
    assert_eq!(v.xwww(), isizevec4(1_isize, 4_isize, 4_isize, 4_isize));
    assert_eq!(v.yxxx(), isizevec4(2_isize, 1_isize, 1_isize, 1_isize));
    assert_eq!(v.yxxy(), isizevec4(2_isize, 1_isize, 1_isize, 2_isize));
    assert_eq!(v.yxxz(), isizevec4(2_isize, 1_isize, 1_isize, 3_isize));
    assert_eq!(v.yxxw(), isizevec4(2_isize, 1_isize, 1_isize, 4_isize));
    assert_eq!(v.yxyx(), isizevec4(2_isize, 1_isize, 2_isize, 1_isize));
    assert_eq!(v.yxyy(), isizevec4(2_isize, 1_isize, 2_isize, 2_isize));
    assert_eq!(v.yxyz(), isizevec4(2_isize, 1_isize, 2_isize, 3_isize));
    assert_eq!(v.yxyw(), isizevec4(2_isize, 1_isize, 2_isize, 4_isize));
    assert_eq!(v.yxzx(), isizevec4(2_isize, 1_isize, 3_isize, 1_isize));
    assert_eq!(v.yxzy(), isizevec4(2_isize, 1_isize, 3_isize, 2_isize));
    assert_eq!(v.yxzz(), isizevec4(2_isize, 1_isize, 3_isize, 3_isize));
    assert_eq!(v.yxzw(), isizevec4(2_isize, 1_isize, 3_isize, 4_isize));
    assert_eq!(v.yxwx(), isizevec4(2_isize, 1_isize, 4_isize, 1_isize));
    assert_eq!(v.yxwy(), isizevec4(2_isize, 1_isize, 4_isize, 2_isize));
    assert_eq!(v.yxwz(), isizevec4(2_isize, 1_isize, 4_isize, 3_isize));
    assert_eq!(v.yxww(), isizevec4(2_isize, 1_isize, 4_isize, 4_isize));
    assert_eq!(v.yyxx(), isizevec4(2_isize, 2_isize, 1_isize, 1_isize));
    assert_eq!(v.yyxy(), isizevec4(2_isize, 2_isize, 1_isize, 2_isize));
    assert_eq!(v.yyxz(), isizevec4(2_isize, 2_isize, 1_isize, 3_isize));
    assert_eq!(v.yyxw(), isizevec4(2_isize, 2_isize, 1_isize, 4_isize));
    assert_eq!(v.yyyx(), isizevec4(2_isize, 2_isize, 2_isize, 1_isize));
    assert_eq!(v.yyyy(), isizevec4(2_isize, 2_isize, 2_isize, 2_isize));
    assert_eq!(v.yyyz(), isizevec4(2_isize, 2_isize, 2_isize, 3_isize));
    assert_eq!(v.yyyw(), isizevec4(2_isize, 2_isize, 2_isize, 4_isize));
    assert_eq!(v.yyzx(), isizevec4(2_isize, 2_isize, 3_isize, 1_isize));
    assert_eq!(v.yyzy(), isizevec4(2_isize, 2_isize, 3_isize, 2_isize));
    assert_eq!(v.yyzz(), isizevec4(2_isize, 2_isize, 3_isize, 3_isize));
    assert_eq!(v.yyzw(), isizevec4(2_isize, 2_isize, 3_isize, 4_isize));
    assert_eq!(v.yywx(), isizevec4(2_isize, 2_isize, 4_isize, 1_isize));
    assert_eq!(v.yywy(), isizevec4(2_isize, 2_isize, 4_isize, 2_isize));
    assert_eq!(v.yywz(), isizevec4(2_isize, 2_isize, 4_isize, 3_isize));
    assert_eq!(v.yyww(), isizevec4(2_isize, 2_isize, 4_isize, 4_isize));
    assert_eq!(v.yzxx(), isizevec4(2_isize, 3_isize, 1_isize, 1_isize));
    assert_eq!(v.yzxy(), isizevec4(2_isize, 3_isize, 1_isize, 2_isize));
    assert_eq!(v.yzxz(), isizevec4(2_isize, 3_isize, 1_isize, 3_isize));
    assert_eq!(v.yzxw(), isizevec4(2_isize, 3_isize, 1_isize, 4_isize));
    assert_eq!(v.yzyx(), isizevec4(2_isize, 3_isize, 2_isize, 1_isize));
    assert_eq!(v.yzyy(), isizevec4(2_isize, 3_isize, 2_isize, 2_isize));
    assert_eq!(v.yzyz(), isizevec4(2_isize, 3_isize, 2_isize, 3_isize));
    assert_eq!(v.yzyw(), isizevec4(2_isize, 3_isize, 2_isize, 4_isize));
    assert_eq!(v.yzzx(), isizevec4(2_isize, 3_isize, 3_isize, 1_isize));
    assert_eq!(v.yzzy(), isizevec4(2_isize, 3_isize, 3_isize, 2_isize));
    assert_eq!(v.yzzz(), isizevec4(2_isize, 3_isize, 3_isize, 3_isize));
    assert_eq!(v.yzzw(), isizevec4(2_isize, 3_isize, 3_isize, 4_isize));
    assert_eq!(v.yzwx(), isizevec4(2_isize, 3_isize, 4_isize, 1_isize));
    assert_eq!(v.yzwy(), isizevec4(2_isize, 3_isize, 4_isize, 2_isize));
    assert_eq!(v.yzwz(), isizevec4(2_isize, 3_isize, 4_isize, 3_isize));
    assert_eq!(v.yzww(), isizevec4(2_isize, 3_isize, 4_isize, 4_isize));
    assert_eq!(v.ywxx(), isizevec4(2_isize, 4_isize, 1_isize, 1_isize));
    assert_eq!(v.ywxy(), isizevec4(2_isize, 4_isize, 1_isize, 2_isize));
    assert_eq!(v.ywxz(), isizevec4(2_isize, 4_isize, 1_isize, 3_isize));
    assert_eq!(v.ywxw(), isizevec4(2_isize, 4_isize, 1_isize, 4_isize));
    assert_eq!(v.ywyx(), isizevec4(2_isize, 4_isize, 2_isize, 1_isize));
    assert_eq!(v.ywyy(), isizevec4(2_isize, 4_isize, 2_isize, 2_isize));
    assert_eq!(v.ywyz(), isizevec4(2_isize, 4_isize, 2_isize, 3_isize));
    assert_eq!(v.ywyw(), isizevec4(2_isize, 4_isize, 2_isize, 4_isize));
    assert_eq!(v.ywzx(), isizevec4(2_isize, 4_isize, 3_isize, 1_isize));
    assert_eq!(v.ywzy(), isizevec4(2_isize, 4_isize, 3_isize, 2_isize));
    assert_eq!(v.ywzz(), isizevec4(2_isize, 4_isize, 3_isize, 3_isize));
    assert_eq!(v.ywzw(), isizevec4(2_isize, 4_isize, 3_isize, 4_isize));
    assert_eq!(v.ywwx(), isizevec4(2_isize, 4_isize, 4_isize, 1_isize));
    assert_eq!(v.ywwy(), isizevec4(2_isize, 4_isize, 4_isize, 2_isize));
    assert_eq!(v.ywwz(), isizevec4(2_isize, 4_isize, 4_isize, 3_isize));
    assert_eq!(v.ywww(), isizevec4(2_isize, 4_isize, 4_isize, 4_isize));
    assert_eq!(v.zxxx(), isizevec4(3_isize, 1_isize, 1_isize, 1_isize));
    assert_eq!(v.zxxy(), isizevec4(3_isize, 1_isize, 1_isize, 2_isize));
    assert_eq!(v.zxxz(), isizevec4(3_isize, 1_isize, 1_isize, 3_isize));
    assert_eq!(v.zxxw(), isizevec4(3_isize, 1_isize, 1_isize, 4_isize));
    assert_eq!(v.zxyx(), isizevec4(3_isize, 1_isize, 2_isize, 1_isize));
    assert_eq!(v.zxyy(), isizevec4(3_isize, 1_isize, 2_isize, 2_isize));
    assert_eq!(v.zxyz(), isizevec4(3_isize, 1_isize, 2_isize, 3_isize));
    assert_eq!(v.zxyw(), isizevec4(3_isize, 1_isize, 2_isize, 4_isize));
    assert_eq!(v.zxzx(), isizevec4(3_isize, 1_isize, 3_isize, 1_isize));
    assert_eq!(v.zxzy(), isizevec4(3_isize, 1_isize, 3_isize, 2_isize));
    assert_eq!(v.zxzz(), isizevec4(3_isize, 1_isize, 3_isize, 3_isize));
    assert_eq!(v.zxzw(), isizevec4(3_isize, 1_isize, 3_isize, 4_isize));
    assert_eq!(v.zxwx(), isizevec4(3_isize, 1_isize, 4_isize, 1_isize));
    assert_eq!(v.zxwy(), isizevec4(3_isize, 1_isize, 4_isize, 2_isize));
    assert_eq!(v.zxwz(), isizevec4(3_isize, 1_isize, 4_isize, 3_isize));
    assert_eq!(v.zxww(), isizevec4(3_isize, 1_isize, 4_isize, 4_isize));
    assert_eq!(v.zyxx(), isizevec4(3_isize, 2_isize, 1_isize, 1_isize));
    assert_eq!(v.zyxy(), isizevec4(3_isize, 2_isize, 1_isize, 2_isize));
    assert_eq!(v.zyxz(), isizevec4(3_isize, 2_isize, 1_isize, 3_isize));
    assert_eq!(v.zyxw(), isizevec4(3_isize, 2_isize, 1_isize, 4_isize));
    assert_eq!(v.zyyx(), isizevec4(3_isize, 2_isize, 2_isize, 1_isize));
    assert_eq!(v.zyyy(), isizevec4(3_isize, 2_isize, 2_isize, 2_isize));
    assert_eq!(v.zyyz(), isizevec4(3_isize, 2_isize, 2_isize, 3_isize));
    assert_eq!(v.zyyw(), isizevec4(3_isize, 2_isize, 2_isize, 4_isize));
    assert_eq!(v.zyzx(), isizevec4(3_isize, 2_isize, 3_isize, 1_isize));
    assert_eq!(v.zyzy(), isizevec4(3_isize, 2_isize, 3_isize, 2_isize));
    assert_eq!(v.zyzz(), isizevec4(3_isize, 2_isize, 3_isize, 3_isize));
    assert_eq!(v.zyzw(), isizevec4(3_isize, 2_isize, 3_isize, 4_isize));
    assert_eq!(v.zywx(), isizevec4(3_isize, 2_isize, 4_isize, 1_isize));
    assert_eq!(v.zywy(), isizevec4(3_isize, 2_isize, 4_isize, 2_isize));
    assert_eq!(v.zywz(), isizevec4(3_isize, 2_isize, 4_isize, 3_isize));
    assert_eq!(v.zyww(), isizevec4(3_isize, 2_isize, 4_isize, 4_isize));
    assert_eq!(v.zzxx(), isizevec4(3_isize, 3_isize, 1_isize, 1_isize));
    assert_eq!(v.zzxy(), isizevec4(3_isize, 3_isize, 1_isize, 2_isize));
    assert_eq!(v.zzxz(), isizevec4(3_isize, 3_isize, 1_isize, 3_isize));
    assert_eq!(v.zzxw(), isizevec4(3_isize, 3_isize, 1_isize, 4_isize));
    assert_eq!(v.zzyx(), isizevec4(3_isize, 3_isize, 2_isize, 1_isize));
    assert_eq!(v.zzyy(), isizevec4(3_isize, 3_isize, 2_isize, 2_isize));
    assert_eq!(v.zzyz(), isizevec4(3_isize, 3_isize, 2_isize, 3_isize));
    assert_eq!(v.zzyw(), isizevec4(3_isize, 3_isize, 2_isize, 4_isize));
    assert_eq!(v.zzzx(), isizevec4(3_isize, 3_isize, 3_isize, 1_isize));
    assert_eq!(v.zzzy(), isizevec4(3_isize, 3_isize, 3_isize, 2_isize));
    assert_eq!(v.zzzz(), isizevec4(3_isize, 3_isize, 3_isize, 3_isize));
    assert_eq!(v.zzzw(), isizevec4(3_isize, 3_isize, 3_isize, 4_isize));
    assert_eq!(v.zzwx(), isizevec4(3_isize, 3_isize, 4_isize, 1_isize));
    assert_eq!(v.zzwy(), isizevec4(3_isize, 3_isize, 4_isize, 2_isize));
    assert_eq!(v.zzwz(), isizevec4(3_isize, 3_isize, 4_isize, 3_isize));
    assert_eq!(v.zzww(), isizevec4(3_isize, 3_isize, 4_isize, 4_isize));
    assert_eq!(v.zwxx(), isizevec4(3_isize, 4_isize, 1_isize, 1_isize));
    assert_eq!(v.zwxy(), isizevec4(3_isize, 4_isize, 1_isize, 2_isize));
    assert_eq!(v.zwxz(), isizevec4(3_isize, 4_isize, 1_isize, 3_isize));
    assert_eq!(v.zwxw(), isizevec4(3_isize, 4_isize, 1_isize, 4_isize));
    assert_eq!(v.zwyx(), isizevec4(3_isize, 4_isize, 2_isize, 1_isize));
    assert_eq!(v.zwyy(), isizevec4(3_isize, 4_isize, 2_isize, 2_isize));
    assert_eq!(v.zwyz(), isizevec4(3_isize, 4_isize, 2_isize, 3_isize));
    assert_eq!(v.zwyw(), isizevec4(3_isize, 4_isize, 2_isize, 4_isize));
    assert_eq!(v.zwzx(), isizevec4(3_isize, 4_isize, 3_isize, 1_isize));
    assert_eq!(v.zwzy(), isizevec4(3_isize, 4_isize, 3_isize, 2_isize));
    assert_eq!(v.zwzz(), isizevec4(3_isize, 4_isize, 3_isize, 3_isize));
    assert_eq!(v.zwzw(), isizevec4(3_isize, 4_isize, 3_isize, 4_isize));
    assert_eq!(v.zwwx(), isizevec4(3_isize, 4_isize, 4_isize, 1_isize));
    assert_eq!(v.zwwy(), isizevec4(3_isize, 4_isize, 4_isize, 2_isize));
    assert_eq!(v.zwwz(), isizevec4(3_isize, 4_isize, 4_isize, 3_isize));
    assert_eq!(v.zwww(), isizevec4(3_isize, 4_isize, 4_isize, 4_isize));
    assert_eq!(v.wxxx(), isizevec4(4_isize, 1_isize, 1_isize, 1_isize));
    assert_eq!(v.wxxy(), isizevec4(4_isize, 1_isize, 1_isize, 2_isize));
    assert_eq!(v.wxxz(), isizevec4(4_isize, 1_isize, 1_isize, 3_isize));
    assert_eq!(v.wxxw(), isizevec4(4_isize, 1_isize, 1_isize, 4_isize));
    assert_eq!(v.wxyx(), isizevec4(4_isize, 1_isize, 2_isize, 1_isize));
    assert_eq!(v.wxyy(), isizevec4(4_isize, 1_isize, 2_isize, 2_isize));
    assert_eq!(v.wxyz(), isizevec4(4_isize, 1_isize, 2_isize, 3_isize));
    assert_eq!(v.wxyw(), isizevec4(4_isize, 1_isize, 2_isize, 4_isize));
    assert_eq!(v.wxzx(), isizevec4(4_isize, 1_isize, 3_isize, 1_isize));
    assert_eq!(v.wxzy(), isizevec4(4_isize, 1_isize, 3_isize, 2_isize));
    assert_eq!(v.wxzz(), isizevec4(4_isize, 1_isize, 3_isize, 3_isize));
    assert_eq!(v.wxzw(), isizevec4(4_isize, 1_isize, 3_isize, 4_isize));
    assert_eq!(v.wxwx(), isizevec4(4_isize, 1_isize, 4_isize, 1_isize));
    assert_eq!(v.wxwy(), isizevec4(4_isize, 1_isize, 4_isize, 2_isize));
    assert_eq!(v.wxwz(), isizevec4(4_isize, 1_isize, 4_isize, 3_isize));
    assert_eq!(v.wxww(), isizevec4(4_isize, 1_isize, 4_isize, 4_isize));
    assert_eq!(v.wyxx(), isizevec4(4_isize, 2_isize, 1_isize, 1_isize));
    assert_eq!(v.wyxy(), isizevec4(4_isize, 2_isize, 1_isize, 2_isize));
    assert_eq!(v.wyxz(), isizevec4(4_isize, 2_isize, 1_isize, 3_isize));
    assert_eq!(v.wyxw(), isizevec4(4_isize, 2_isize, 1_isize, 4_isize));
    assert_eq!(v.wyyx(), isizevec4(4_isize, 2_isize, 2_isize, 1_isize));
    assert_eq!(v.wyyy(), isizevec4(4_isize, 2_isize, 2_isize, 2_isize));
    assert_eq!(v.wyyz(), isizevec4(4_isize, 2_isize, 2_isize, 3_isize));
    assert_eq!(v.wyyw(), isizevec4(4_isize, 2_isize, 2_isize, 4_isize));
    assert_eq!(v.wyzx(), isizevec4(4_isize, 2_isize, 3_isize, 1_isize));
    assert_eq!(v.wyzy(), isizevec4(4_isize, 2_isize, 3_isize, 2_isize));
    assert_eq!(v.wyzz(), isizevec4(4_isize, 2_isize, 3_isize, 3_isize));
    assert_eq!(v.wyzw(), isizevec4(4_isize, 2_isize, 3_isize, 4_isize));
    assert_eq!(v.wywx(), isizevec4(4_isize, 2_isize, 4_isize, 1_isize));
    assert_eq!(v.wywy(), isizevec4(4_isize, 2_isize, 4_isize, 2_isize));
    assert_eq!(v.wywz(), isizevec4(4_isize, 2_isize, 4_isize, 3_isize));
    assert_eq!(v.wyww(), isizevec4(4_isize, 2_isize, 4_isize, 4_isize));
    assert_eq!(v.wzxx(), isizevec4(4_isize, 3_isize, 1_isize, 1_isize));
    assert_eq!(v.wzxy(), isizevec4(4_isize, 3_isize, 1_isize, 2_isize));
    assert_eq!(v.wzxz(), isizevec4(4_isize, 3_isize, 1_isize, 3_isize));
    assert_eq!(v.wzxw(), isizevec4(4_isize, 3_isize, 1_isize, 4_isize));
    assert_eq!(v.wzyx(), isizevec4(4_isize, 3_isize, 2_isize, 1_isize));
    assert_eq!(v.wzyy(), isizevec4(4_isize, 3_isize, 2_isize, 2_isize));
    assert_eq!(v.wzyz(), isizevec4(4_isize, 3_isize, 2_isize, 3_isize));
    assert_eq!(v.wzyw(), isizevec4(4_isize, 3_isize, 2_isize, 4_isize));
    assert_eq!(v.wzzx(), isizevec4(4_isize, 3_isize, 3_isize, 1_isize));
    assert_eq!(v.wzzy(), isizevec4(4_isize, 3_isize, 3_isize, 2_isize));
    assert_eq!(v.wzzz(), isizevec4(4_isize, 3_isize, 3_isize, 3_isize));
    assert_eq!(v.wzzw(), isizevec4(4_isize, 3_isize, 3_isize, 4_isize));
    assert_eq!(v.wzwx(), isizevec4(4_isize, 3_isize, 4_isize, 1_isize));
    assert_eq!(v.wzwy(), isizevec4(4_isize, 3_isize, 4_isize, 2_isize));
    assert_eq!(v.wzwz(), isizevec4(4_isize, 3_isize, 4_isize, 3_isize));
    assert_eq!(v.wzww(), isizevec4(4_isize, 3_isize, 4_isize, 4_isize));
    assert_eq!(v.wwxx(), isizevec4(4_isize, 4_isize, 1_isize, 1_isize));
    assert_eq!(v.wwxy(), isizevec4(4_isize, 4_isize, 1_isize, 2_isize));
    assert_eq!(v.wwxz(), isizevec4(4_isize, 4_isize, 1_isize, 3_isize));
    assert_eq!(v.wwxw(), isizevec4(4_isize, 4_isize, 1_isize, 4_isize));
    assert_eq!(v.wwyx(), isizevec4(4_isize, 4_isize, 2_isize, 1_isize));
    assert_eq!(v.wwyy(), isizevec4(4_isize, 4_isize, 2_isize, 2_isize));
    assert_eq!(v.wwyz(), isizevec4(4_isize, 4_isize, 2_isize, 3_isize));
    assert_eq!(v.wwyw(), isizevec4(4_isize, 4_isize, 2_isize, 4_isize));
    assert_eq!(v.wwzx(), isizevec4(4_isize, 4_isize, 3_isize, 1_isize));
    assert_eq!(v.wwzy(), isizevec4(4_isize, 4_isize, 3_isize, 2_isize));
    assert_eq!(v.wwzz(), isizevec4(4_isize, 4_isize, 3_isize, 3_isize));
    assert_eq!(v.wwzw(), isizevec4(4_isize, 4_isize, 3_isize, 4_isize));
    assert_eq!(v.wwwx(), isizevec4(4_isize, 4_isize, 4_isize, 1_isize));
    assert_eq!(v.wwwy(), isizevec4(4_isize, 4_isize, 4_isize, 2_isize));
    assert_eq!(v.wwwz(), isizevec4(4_isize, 4_isize, 4_isize, 3_isize));
    assert_eq!(v.wwww(), isizevec4(4_isize, 4_isize, 4_isize, 4_isize));
    assert_eq!(v.xxx(), isizevec3(1_isize, 1_isize, 1_isize));
    assert_eq!(v.xxy(), isizevec3(1_isize, 1_isize, 2_isize));
    assert_eq!(v.xxz(), isizevec3(1_isize, 1_isize, 3_isize));
    assert_eq!(v.xxw(), isizevec3(1_isize, 1_isize, 4_isize));
    assert_eq!(v.xyx(), isizevec3(1_isize, 2_isize, 1_isize));
    assert_eq!(v.xyy(), isizevec3(1_isize, 2_isize, 2_isize));
    assert_eq!(v.xyz(), isizevec3(1_isize, 2_isize, 3_isize));
    assert_eq!(v.xyw(), isizevec3(1_isize, 2_isize, 4_isize));
    assert_eq!(v.xzx(), isizevec3(1_isize, 3_isize, 1_isize));
    assert_eq!(v.xzy(), isizevec3(1_isize, 3_isize, 2_isize));
    assert_eq!(v.xzz(), isizevec3(1_isize, 3_isize, 3_isize));
    assert_eq!(v.xzw(), isizevec3(1_isize, 3_isize, 4_isize));
    assert_eq!(v.xwx(), isizevec3(1_isize, 4_isize, 1_isize));
    assert_eq!(v.xwy(), isizevec3(1_isize, 4_isize, 2_isize));
    assert_eq!(v.xwz(), isizevec3(1_isize, 4_isize, 3_isize));
    assert_eq!(v.xww(), isizevec3(1_isize, 4_isize, 4_isize));
    assert_eq!(v.yxx(), isizevec3(2_isize, 1_isize, 1_isize));
    assert_eq!(v.yxy(), isizevec3(2_isize, 1_isize, 2_isize));
    assert_eq!(v.yxz(), isizevec3(2_isize, 1_isize, 3_isize));
    assert_eq!(v.yxw(), isizevec3(2_isize, 1_isize, 4_isize));
    assert_eq!(v.yyx(), isizevec3(2_isize, 2_isize, 1_isize));
    assert_eq!(v.yyy(), isizevec3(2_isize, 2_isize, 2_isize));
    assert_eq!(v.yyz(), isizevec3(2_isize, 2_isize, 3_isize));
    assert_eq!(v.yyw(), isizevec3(2_isize, 2_isize, 4_isize));
    assert_eq!(v.yzx(), isizevec3(2_isize, 3_isize, 1_isize));
    assert_eq!(v.yzy(), isizevec3(2_isize, 3_isize, 2_isize));
    assert_eq!(v.yzz(), isizevec3(2_isize, 3_isize, 3_isize));
    assert_eq!(v.yzw(), isizevec3(2_isize, 3_isize, 4_isize));
    assert_eq!(v.ywx(), isizevec3(2_isize, 4_isize, 1_isize));
    assert_eq!(v.ywy(), isizevec3(2_isize, 4_isize, 2_isize));
    assert_eq!(v.ywz(), isizevec3(2_isize, 4_isize, 3_isize));
    assert_eq!(v.yww(), isizevec3(2_isize, 4_isize, 4_isize));
    assert_eq!(v.zxx(), isizevec3(3_isize, 1_isize, 1_isize));
    assert_eq!(v.zxy(), isizevec3(3_isize, 1_isize, 2_isize));
    assert_eq!(v.zxz(), isizevec3(3_isize, 1_isize, 3_isize));
    assert_eq!(v.zxw(), isizevec3(3_isize, 1_isize, 4_isize));
    assert_eq!(v.zyx(), isizevec3(3_isize, 2_isize, 1_isize));
    assert_eq!(v.zyy(), isizevec3(3_isize, 2_isize, 2_isize));
    assert_eq!(v.zyz(), isizevec3(3_isize, 2_isize, 3_isize));
    assert_eq!(v.zyw(), isizevec3(3_isize, 2_isize, 4_isize));
    assert_eq!(v.zzx(), isizevec3(3_isize, 3_isize, 1_isize));
    assert_eq!(v.zzy(), isizevec3(3_isize, 3_isize, 2_isize));
    assert_eq!(v.zzz(), isizevec3(3_isize, 3_isize, 3_isize));
    assert_eq!(v.zzw(), isizevec3(3_isize, 3_isize, 4_isize));
    assert_eq!(v.zwx(), isizevec3(3_isize, 4_isize, 1_isize));
    assert_eq!(v.zwy(), isizevec3(3_isize, 4_isize, 2_isize));
    assert_eq!(v.zwz(), isizevec3(3_isize, 4_isize, 3_isize));
    assert_eq!(v.zww(), isizevec3(3_isize, 4_isize, 4_isize));
    assert_eq!(v.wxx(), isizevec3(4_isize, 1_isize, 1_isize));
    assert_eq!(v.wxy(), isizevec3(4_isize, 1_isize, 2_isize));
    assert_eq!(v.wxz(), isizevec3(4_isize, 1_isize, 3_isize));
    assert_eq!(v.wxw(), isizevec3(4_isize, 1_isize, 4_isize));
    assert_eq!(v.wyx(), isizevec3(4_isize, 2_isize, 1_isize));
    assert_eq!(v.wyy(), isizevec3(4_isize, 2_isize, 2_isize));
    assert_eq!(v.wyz(), isizevec3(4_isize, 2_isize, 3_isize));
    assert_eq!(v.wyw(), isizevec3(4_isize, 2_isize, 4_isize));
    assert_eq!(v.wzx(), isizevec3(4_isize, 3_isize, 1_isize));
    assert_eq!(v.wzy(), isizevec3(4_isize, 3_isize, 2_isize));
    assert_eq!(v.wzz(), isizevec3(4_isize, 3_isize, 3_isize));
    assert_eq!(v.wzw(), isizevec3(4_isize, 3_isize, 4_isize));
    assert_eq!(v.wwx(), isizevec3(4_isize, 4_isize, 1_isize));
    assert_eq!(v.wwy(), isizevec3(4_isize, 4_isize, 2_isize));
    assert_eq!(v.wwz(), isizevec3(4_isize, 4_isize, 3_isize));
    assert_eq!(v.www(), isizevec3(4_isize, 4_isize, 4_isize));
    assert_eq!(
        v.with_xyz(rhs3),
        isizevec4(11_isize, 12_isize, 13_isize, 4_isize)
    );
    assert_eq!(
        v.with_xyw(rhs3),
        isizevec4(11_isize, 12_isize, 3_isize, 13_isize)
    );
    assert_eq!(
        v.with_xzy(rhs3),
        isizevec4(11_isize, 13_isize, 12_isize, 4_isize)
    );
    assert_eq!(
        v.with_xzw(rhs3),
        isizevec4(11_isize, 2_isize, 12_isize, 13_isize)
    );
    assert_eq!(
        v.with_xwy(rhs3),
        isizevec4(11_isize, 13_isize, 3_isize, 12_isize)
    );
    assert_eq!(
        v.with_xwz(rhs3),
        isizevec4(11_isize, 2_isize, 13_isize, 12_isize)
    );
    assert_eq!(
        v.with_yxz(rhs3),
        isizevec4(12_isize, 11_isize, 13_isize, 4_isize)
    );
    assert_eq!(
        v.with_yxw(rhs3),
        isizevec4(12_isize, 11_isize, 3_isize, 13_isize)
    );
    assert_eq!(
        v.with_yzx(rhs3),
        isizevec4(13_isize, 11_isize, 12_isize, 4_isize)
    );
    assert_eq!(
        v.with_yzw(rhs3),
        isizevec4(1_isize, 11_isize, 12_isize, 13_isize)
    );
    assert_eq!(
        v.with_ywx(rhs3),
        isizevec4(13_isize, 11_isize, 3_isize, 12_isize)
    );
    assert_eq!(
        v.with_ywz(rhs3),
        isizevec4(1_isize, 11_isize, 13_isize, 12_isize)
    );
    assert_eq!(
        v.with_zxy(rhs3),
        isizevec4(12_isize, 13_isize, 11_isize, 4_isize)
    );
    assert_eq!(
        v.with_zxw(rhs3),
        isizevec4(12_isize, 2_isize, 11_isize, 13_isize)
    );
    assert_eq!(
        v.with_zyx(rhs3),
        isizevec4(13_isize, 12_isize, 11_isize, 4_isize)
    );
    assert_eq!(
        v.with_zyw(rhs3),
        isizevec4(1_isize, 12_isize, 11_isize, 13_isize)
    );
    assert_eq!(
        v.with_zwx(rhs3),
        isizevec4(13_isize, 2_isize, 11_isize, 12_isize)
    );
    assert_eq!(
        v.with_zwy(rhs3),
        isizevec4(1_isize, 13_isize, 11_isize, 12_isize)
    );
    assert_eq!(
        v.with_wxy(rhs3),
        isizevec4(12_isize, 13_isize, 3_isize, 11_isize)
    );
    assert_eq!(
        v.with_wxz(rhs3),
        isizevec4(12_isize, 2_isize, 13_isize, 11_isize)
    );
    assert_eq!(
        v.with_wyx(rhs3),
        isizevec4(13_isize, 12_isize, 3_isize, 11_isize)
    );
    assert_eq!(
        v.with_wyz(rhs3),
        isizevec4(1_isize, 12_isize, 13_isize, 11_isize)
    );
    assert_eq!(
        v.with_wzx(rhs3),
        isizevec4(13_isize, 2_isize, 12_isize, 11_isize)
    );
    assert_eq!(
        v.with_wzy(rhs3),
        isizevec4(1_isize, 13_isize, 12_isize, 11_isize)
    );
    assert_eq!(v.xx(), isizevec2(1_isize, 1_isize));
    assert_eq!(v.xy(), isizevec2(1_isize, 2_isize));
    assert_eq!(v.xz(), isizevec2(1_isize, 3_isize));
    assert_eq!(v.xw(), isizevec2(1_isize, 4_isize));
    assert_eq!(v.yx(), isizevec2(2_isize, 1_isize));
    assert_eq!(v.yy(), isizevec2(2_isize, 2_isize));
    assert_eq!(v.yz(), isizevec2(2_isize, 3_isize));
    assert_eq!(v.yw(), isizevec2(2_isize, 4_isize));
    assert_eq!(v.zx(), isizevec2(3_isize, 1_isize));
    assert_eq!(v.zy(), isizevec2(3_isize, 2_isize));
    assert_eq!(v.zz(), isizevec2(3_isize, 3_isize));
    assert_eq!(v.zw(), isizevec2(3_isize, 4_isize));
    assert_eq!(v.wx(), isizevec2(4_isize, 1_isize));
    assert_eq!(v.wy(), isizevec2(4_isize, 2_isize));
    assert_eq!(v.wz(), isizevec2(4_isize, 3_isize));
    assert_eq!(v.ww(), isizevec2(4_isize, 4_isize));
    assert_eq!(
        v.with_xy(rhs2),
        isizevec4(11_isize, 12_isize, 3_isize, 4_isize)
    );
    assert_eq!(
        v.with_xz(rhs2),
        isizevec4(11_isize, 2_isize, 12_isize, 4_isize)
    );
    assert_eq!(
        v.with_xw(rhs2),
        isizevec4(11_isize, 2_isize, 3_isize, 12_isize)
    );
    assert_eq!(
        v.with_yx(rhs2),
        isizevec4(12_isize, 11_isize, 3_isize, 4_isize)
    );
    assert_eq!(
        v.with_yz(rhs2),
        isizevec4(1_isize, 11_isize, 12_isize, 4_isize)
    );
    assert_eq!(
        v.with_yw(rhs2),
        isizevec4(1_isize, 11_isize, 3_isize, 12_isize)
    );
    assert_eq!(
        v.with_zx(rhs2),
        isizevec4(12_isize, 2_isize, 11_isize, 4_isize)
    );
    assert_eq!(
        v.with_zy(rhs2),
        isizevec4(1_isize, 12_isize, 11_isize, 4_isize)
    );
    assert_eq!(
        v.with_zw(rhs2),
        isizevec4(1_isize, 2_isize, 11_isize, 12_isize)
    );
    assert_eq!(
        v.with_wx(rhs2),
        isizevec4(12_isize, 2_isize, 3_isize, 11_isize)
    );
    assert_eq!(
        v.with_wy(rhs2),
        isizevec4(1_isize, 12_isize, 3_isize, 11_isize)
    );
    assert_eq!(
        v.with_wz(rhs2),
        isizevec4(1_isize, 2_isize, 12_isize, 11_isize)
    );
});

glam_test!(test_isizevec3_swizzles, {
    let v = isizevec3(1_isize, 2_isize, 3_isize);
    let rhs2 = isizevec2(11_isize, 12_isize);
    assert_eq!(v, v.xyz());
    assert_eq!(v.xxxx(), isizevec4(1_isize, 1_isize, 1_isize, 1_isize));
    assert_eq!(v.xxxy(), isizevec4(1_isize, 1_isize, 1_isize, 2_isize));
    assert_eq!(v.xxxz(), isizevec4(1_isize, 1_isize, 1_isize, 3_isize));
    assert_eq!(v.xxyx(), isizevec4(1_isize, 1_isize, 2_isize, 1_isize));
    assert_eq!(v.xxyy(), isizevec4(1_isize, 1_isize, 2_isize, 2_isize));
    assert_eq!(v.xxyz(), isizevec4(1_isize, 1_isize, 2_isize, 3_isize));
    assert_eq!(v.xxzx(), isizevec4(1_isize, 1_isize, 3_isize, 1_isize));
    assert_eq!(v.xxzy(), isizevec4(1_isize, 1_isize, 3_isize, 2_isize));
    assert_eq!(v.xxzz(), isizevec4(1_isize, 1_isize, 3_isize, 3_isize));
    assert_eq!(v.xyxx(), isizevec4(1_isize, 2_isize, 1_isize, 1_isize));
    assert_eq!(v.xyxy(), isizevec4(1_isize, 2_isize, 1_isize, 2_isize));
    assert_eq!(v.xyxz(), isizevec4(1_isize, 2_isize, 1_isize, 3_isize));
    assert_eq!(v.xyyx(), isizevec4(1_isize, 2_isize, 2_isize, 1_isize));
    assert_eq!(v.xyyy(), isizevec4(1_isize, 2_isize, 2_isize, 2_isize));
    assert_eq!(v.xyyz(), isizevec4(1_isize, 2_isize, 2_isize, 3_isize));
    assert_eq!(v.xyzx(), isizevec4(1_isize, 2_isize, 3_isize, 1_isize));
    assert_eq!(v.xyzy(), isizevec4(1_isize, 2_isize, 3_isize, 2_isize));
    assert_eq!(v.xyzz(), isizevec4(1_isize, 2_isize, 3_isize, 3_isize));
    assert_eq!(v.xzxx(), isizevec4(1_isize, 3_isize, 1_isize, 1_isize));
    assert_eq!(v.xzxy(), isizevec4(1_isize, 3_isize, 1_isize, 2_isize));
    assert_eq!(v.xzxz(), isizevec4(1_isize, 3_isize, 1_isize, 3_isize));
    assert_eq!(v.xzyx(), isizevec4(1_isize, 3_isize, 2_isize, 1_isize));
    assert_eq!(v.xzyy(), isizevec4(1_isize, 3_isize, 2_isize, 2_isize));
    assert_eq!(v.xzyz(), isizevec4(1_isize, 3_isize, 2_isize, 3_isize));
    assert_eq!(v.xzzx(), isizevec4(1_isize, 3_isize, 3_isize, 1_isize));
    assert_eq!(v.xzzy(), isizevec4(1_isize, 3_isize, 3_isize, 2_isize));
    assert_eq!(v.xzzz(), isizevec4(1_isize, 3_isize, 3_isize, 3_isize));
    assert_eq!(v.yxxx(), isizevec4(2_isize, 1_isize, 1_isize, 1_isize));
    assert_eq!(v.yxxy(), isizevec4(2_isize, 1_isize, 1_isize, 2_isize));
    assert_eq!(v.yxxz(), isizevec4(2_isize, 1_isize, 1_isize, 3_isize));
    assert_eq!(v.yxyx(), isizevec4(2_isize, 1_isize, 2_isize, 1_isize));
    assert_eq!(v.yxyy(), isizevec4(2_isize, 1_isize, 2_isize, 2_isize));
    assert_eq!(v.yxyz(), isizevec4(2_isize, 1_isize, 2_isize, 3_isize));
    assert_eq!(v.yxzx(), isizevec4(2_isize, 1_isize, 3_isize, 1_isize));
    assert_eq!(v.yxzy(), isizevec4(2_isize, 1_isize, 3_isize, 2_isize));
    assert_eq!(v.yxzz(), isizevec4(2_isize, 1_isize, 3_isize, 3_isize));
    assert_eq!(v.yyxx(), isizevec4(2_isize, 2_isize, 1_isize, 1_isize));
    assert_eq!(v.yyxy(), isizevec4(2_isize, 2_isize, 1_isize, 2_isize));
    assert_eq!(v.yyxz(), isizevec4(2_isize, 2_isize, 1_isize, 3_isize));
    assert_eq!(v.yyyx(), isizevec4(2_isize, 2_isize, 2_isize, 1_isize));
    assert_eq!(v.yyyy(), isizevec4(2_isize, 2_isize, 2_isize, 2_isize));
    assert_eq!(v.yyyz(), isizevec4(2_isize, 2_isize, 2_isize, 3_isize));
    assert_eq!(v.yyzx(), isizevec4(2_isize, 2_isize, 3_isize, 1_isize));
    assert_eq!(v.yyzy(), isizevec4(2_isize, 2_isize, 3_isize, 2_isize));
    assert_eq!(v.yyzz(), isizevec4(2_isize, 2_isize, 3_isize, 3_isize));
    assert_eq!(v.yzxx(), isizevec4(2_isize, 3_isize, 1_isize, 1_isize));
    assert_eq!(v.yzxy(), isizevec4(2_isize, 3_isize, 1_isize, 2_isize));
    assert_eq!(v.yzxz(), isizevec4(2_isize, 3_isize, 1_isize, 3_isize));
    assert_eq!(v.yzyx(), isizevec4(2_isize, 3_isize, 2_isize, 1_isize));
    assert_eq!(v.yzyy(), isizevec4(2_isize, 3_isize, 2_isize, 2_isize));
    assert_eq!(v.yzyz(), isizevec4(2_isize, 3_isize, 2_isize, 3_isize));
    assert_eq!(v.yzzx(), isizevec4(2_isize, 3_isize, 3_isize, 1_isize));
    assert_eq!(v.yzzy(), isizevec4(2_isize, 3_isize, 3_isize, 2_isize));
    assert_eq!(v.yzzz(), isizevec4(2_isize, 3_isize, 3_isize, 3_isize));
    assert_eq!(v.zxxx(), isizevec4(3_isize, 1_isize, 1_isize, 1_isize));
    assert_eq!(v.zxxy(), isizevec4(3_isize, 1_isize, 1_isize, 2_isize));
    assert_eq!(v.zxxz(), isizevec4(3_isize, 1_isize, 1_isize, 3_isize));
    assert_eq!(v.zxyx(), isizevec4(3_isize, 1_isize, 2_isize, 1_isize));
    assert_eq!(v.zxyy(), isizevec4(3_isize, 1_isize, 2_isize, 2_isize));
    assert_eq!(v.zxyz(), isizevec4(3_isize, 1_isize, 2_isize, 3_isize));
    assert_eq!(v.zxzx(), isizevec4(3_isize, 1_isize, 3_isize, 1_isize));
    assert_eq!(v.zxzy(), isizevec4(3_isize, 1_isize, 3_isize, 2_isize));
    assert_eq!(v.zxzz(), isizevec4(3_isize, 1_isize, 3_isize, 3_isize));
    assert_eq!(v.zyxx(), isizevec4(3_isize, 2_isize, 1_isize, 1_isize));
    assert_eq!(v.zyxy(), isizevec4(3_isize, 2_isize, 1_isize, 2_isize));
    assert_eq!(v.zyxz(), isizevec4(3_isize, 2_isize, 1_isize, 3_isize));
    assert_eq!(v.zyyx(), isizevec4(3_isize, 2_isize, 2_isize, 1_isize));
    assert_eq!(v.zyyy(), isizevec4(3_isize, 2_isize, 2_isize, 2_isize));
    assert_eq!(v.zyyz(), isizevec4(3_isize, 2_isize, 2_isize, 3_isize));
    assert_eq!(v.zyzx(), isizevec4(3_isize, 2_isize, 3_isize, 1_isize));
    assert_eq!(v.zyzy(), isizevec4(3_isize, 2_isize, 3_isize, 2_isize));
    assert_eq!(v.zyzz(), isizevec4(3_isize, 2_isize, 3_isize, 3_isize));
    assert_eq!(v.zzxx(), isizevec4(3_isize, 3_isize, 1_isize, 1_isize));
    assert_eq!(v.zzxy(), isizevec4(3_isize, 3_isize, 1_isize, 2_isize));
    assert_eq!(v.zzxz(), isizevec4(3_isize, 3_isize, 1_isize, 3_isize));
    assert_eq!(v.zzyx(), isizevec4(3_isize, 3_isize, 2_isize, 1_isize));
    assert_eq!(v.zzyy(), isizevec4(3_isize, 3_isize, 2_isize, 2_isize));
    assert_eq!(v.zzyz(), isizevec4(3_isize, 3_isize, 2_isize, 3_isize));
    assert_eq!(v.zzzx(), isizevec4(3_isize, 3_isize, 3_isize, 1_isize));
    assert_eq!(v.zzzy(), isizevec4(3_isize, 3_isize, 3_isize, 2_isize));
    assert_eq!(v.zzzz(), isizevec4(3_isize, 3_isize, 3_isize, 3_isize));
    assert_eq!(v.xxx(), isizevec3(1_isize, 1_isize, 1_isize));
    assert_eq!(v.xxy(), isizevec3(1_isize, 1_isize, 2_isize));
    assert_eq!(v.xxz(), isizevec3(1_isize, 1_isize, 3_isize));
    assert_eq!(v.xyx(), isizevec3(1_isize, 2_isize, 1_isize));
    assert_eq!(v.xyy(), isizevec3(1_isize, 2_isize, 2_isize));
    assert_eq!(v.xzx(), isizevec3(1_isize, 3_isize, 1_isize));
    assert_eq!(v.xzy(), isizevec3(1_isize, 3_isize, 2_isize));
    assert_eq!(v.xzz(), isizevec3(1_isize, 3_isize, 3_isize));
    assert_eq!(v.yxx(), isizevec3(2_isize, 1_isize, 1_isize));
    assert_eq!(v.yxy(), isizevec3(2_isize, 1_isize, 2_isize));
    assert_eq!(v.yxz(), isizevec3(2_isize, 1_isize, 3_isize));
    assert_eq!(v.yyx(), isizevec3(2_isize, 2_isize, 1_isize));
    assert_eq!(v.yyy(), isizevec3(2_isize, 2_isize, 2_isize));
    assert_eq!(v.yyz(), isizevec3(2_isize, 2_isize, 3_isize));
    assert_eq!(v.yzx(), isizevec3(2_isize, 3_isize, 1_isize));
    assert_eq!(v.yzy(), isizevec3(2_isize, 3_isize, 2_isize));
    assert_eq!(v.yzz(), isizevec3(2_isize, 3_isize, 3_isize));
    assert_eq!(v.zxx(), isizevec3(3_isize, 1_isize, 1_isize));
    assert_eq!(v.zxy(), isizevec3(3_isize, 1_isize, 2_isize));
    assert_eq!(v.zxz(), isizevec3(3_isize, 1_isize, 3_isize));
    assert_eq!(v.zyx(), isizevec3(3_isize, 2_isize, 1_isize));
    assert_eq!(v.zyy(), isizevec3(3_isize, 2_isize, 2_isize));
    assert_eq!(v.zyz(), isizevec3(3_isize, 2_isize, 3_isize));
    assert_eq!(v.zzx(), isizevec3(3_isize, 3_isize, 1_isize));
    assert_eq!(v.zzy(), isizevec3(3_isize, 3_isize, 2_isize));
    assert_eq!(v.zzz(), isizevec3(3_isize, 3_isize, 3_isize));
    assert_eq!(v.xx(), isizevec2(1_isize, 1_isize));
    assert_eq!(v.xy(), isizevec2(1_isize, 2_isize));
    assert_eq!(v.xz(), isizevec2(1_isize, 3_isize));
    assert_eq!(v.yx(), isizevec2(2_isize, 1_isize));
    assert_eq!(v.yy(), isizevec2(2_isize, 2_isize));
    assert_eq!(v.yz(), isizevec2(2_isize, 3_isize));
    assert_eq!(v.zx(), isizevec2(3_isize, 1_isize));
    assert_eq!(v.zy(), isizevec2(3_isize, 2_isize));
    assert_eq!(v.zz(), isizevec2(3_isize, 3_isize));
    assert_eq!(v.with_xy(rhs2), isizevec3(11_isize, 12_isize, 3_isize));
    assert_eq!(v.with_xz(rhs2), isizevec3(11_isize, 2_isize, 12_isize));
    assert_eq!(v.with_yx(rhs2), isizevec3(12_isize, 11_isize, 3_isize));
    assert_eq!(v.with_yz(rhs2), isizevec3(1_isize, 11_isize, 12_isize));
    assert_eq!(v.with_zx(rhs2), isizevec3(12_isize, 2_isize, 11_isize));
    assert_eq!(v.with_zy(rhs2), isizevec3(1_isize, 12_isize, 11_isize));
});

glam_test!(test_isizevec2_swizzles, {
    let v = isizevec2(1_isize, 2_isize);
    assert_eq!(v, v.xy());
    assert_eq!(v.xxxx(), isizevec4(1_isize, 1_isize, 1_isize, 1_isize));
    assert_eq!(v.xxxy(), isizevec4(1_isize, 1_isize, 1_isize, 2_isize));
    assert_eq!(v.xxyx(), isizevec4(1_isize, 1_isize, 2_isize, 1_isize));
    assert_eq!(v.xxyy(), isizevec4(1_isize, 1_isize, 2_isize, 2_isize));
    assert_eq!(v.xyxx(), isizevec4(1_isize, 2_isize, 1_isize, 1_isize));
    assert_eq!(v.xyxy(), isizevec4(1_isize, 2_isize, 1_isize, 2_isize));
    assert_eq!(v.xyyx(), isizevec4(1_isize, 2_isize, 2_isize, 1_isize));
    assert_eq!(v.xyyy(), isizevec4(1_isize, 2_isize, 2_isize, 2_isize));
    assert_eq!(v.yxxx(), isizevec4(2_isize, 1_isize, 1_isize, 1_isize));
    assert_eq!(v.yxxy(), isizevec4(2_isize, 1_isize, 1_isize, 2_isize));
    assert_eq!(v.yxyx(), isizevec4(2_isize, 1_isize, 2_isize, 1_isize));
    assert_eq!(v.yxyy(), isizevec4(2_isize, 1_isize, 2_isize, 2_isize));
    assert_eq!(v.yyxx(), isizevec4(2_isize, 2_isize, 1_isize, 1_isize));
    assert_eq!(v.yyxy(), isizevec4(2_isize, 2_isize, 1_isize, 2_isize));
    assert_eq!(v.yyyx(), isizevec4(2_isize, 2_isize, 2_isize, 1_isize));
    assert_eq!(v.yyyy(), isizevec4(2_isize, 2_isize, 2_isize, 2_isize));
    assert_eq!(v.xxx(), isizevec3(1_isize, 1_isize, 1_isize));
    assert_eq!(v.xxy(), isizevec3(1_isize, 1_isize, 2_isize));
    assert_eq!(v.xyx(), isizevec3(1_isize, 2_isize, 1_isize));
    assert_eq!(v.xyy(), isizevec3(1_isize, 2_isize, 2_isize));
    assert_eq!(v.yxx(), isizevec3(2_isize, 1_isize, 1_isize));
    assert_eq!(v.yxy(), isizevec3(2_isize, 1_isize, 2_isize));
    assert_eq!(v.yyx(), isizevec3(2_isize, 2_isize, 1_isize));
    assert_eq!(v.yyy(), isizevec3(2_isize, 2_isize, 2_isize));
    assert_eq!(v.xx(), isizevec2(1_isize, 1_isize));
    assert_eq!(v.yx(), isizevec2(2_isize, 1_isize));
    assert_eq!(v.yy(), isizevec2(2_isize, 2_isize));
});
