// Generated by swizzlegen. Do not edit.
#[macro_use]
mod support;
use glam::*;

glam_test!(test_usizevec4_swizzles, {
    let v = usizevec4(1_usize, 2_usize, 3_usize, 4_usize);
    let rhs3 = usizevec3(11_usize, 12_usize, 13_usize);
    let rhs2 = usizevec2(11_usize, 12_usize);
    assert_eq!(v, v.xyzw());
    assert_eq!(v.xxxx(), usizevec4(1_usize, 1_usize, 1_usize, 1_usize));
    assert_eq!(v.xxxy(), usizevec4(1_usize, 1_usize, 1_usize, 2_usize));
    assert_eq!(v.xxxz(), usizevec4(1_usize, 1_usize, 1_usize, 3_usize));
    assert_eq!(v.xxxw(), usizevec4(1_usize, 1_usize, 1_usize, 4_usize));
    assert_eq!(v.xxyx(), usizevec4(1_usize, 1_usize, 2_usize, 1_usize));
    assert_eq!(v.xxyy(), usizevec4(1_usize, 1_usize, 2_usize, 2_usize));
    assert_eq!(v.xxyz(), usizevec4(1_usize, 1_usize, 2_usize, 3_usize));
    assert_eq!(v.xxyw(), usizevec4(1_usize, 1_usize, 2_usize, 4_usize));
    assert_eq!(v.xxzx(), usizevec4(1_usize, 1_usize, 3_usize, 1_usize));
    assert_eq!(v.xxzy(), usizevec4(1_usize, 1_usize, 3_usize, 2_usize));
    assert_eq!(v.xxzz(), usizevec4(1_usize, 1_usize, 3_usize, 3_usize));
    assert_eq!(v.xxzw(), usizevec4(1_usize, 1_usize, 3_usize, 4_usize));
    assert_eq!(v.xxwx(), usizevec4(1_usize, 1_usize, 4_usize, 1_usize));
    assert_eq!(v.xxwy(), usizevec4(1_usize, 1_usize, 4_usize, 2_usize));
    assert_eq!(v.xxwz(), usizevec4(1_usize, 1_usize, 4_usize, 3_usize));
    assert_eq!(v.xxww(), usizevec4(1_usize, 1_usize, 4_usize, 4_usize));
    assert_eq!(v.xyxx(), usizevec4(1_usize, 2_usize, 1_usize, 1_usize));
    assert_eq!(v.xyxy(), usizevec4(1_usize, 2_usize, 1_usize, 2_usize));
    assert_eq!(v.xyxz(), usizevec4(1_usize, 2_usize, 1_usize, 3_usize));
    assert_eq!(v.xyxw(), usizevec4(1_usize, 2_usize, 1_usize, 4_usize));
    assert_eq!(v.xyyx(), usizevec4(1_usize, 2_usize, 2_usize, 1_usize));
    assert_eq!(v.xyyy(), usizevec4(1_usize, 2_usize, 2_usize, 2_usize));
    assert_eq!(v.xyyz(), usizevec4(1_usize, 2_usize, 2_usize, 3_usize));
    assert_eq!(v.xyyw(), usizevec4(1_usize, 2_usize, 2_usize, 4_usize));
    assert_eq!(v.xyzx(), usizevec4(1_usize, 2_usize, 3_usize, 1_usize));
    assert_eq!(v.xyzy(), usizevec4(1_usize, 2_usize, 3_usize, 2_usize));
    assert_eq!(v.xyzz(), usizevec4(1_usize, 2_usize, 3_usize, 3_usize));
    assert_eq!(v.xywx(), usizevec4(1_usize, 2_usize, 4_usize, 1_usize));
    assert_eq!(v.xywy(), usizevec4(1_usize, 2_usize, 4_usize, 2_usize));
    assert_eq!(v.xywz(), usizevec4(1_usize, 2_usize, 4_usize, 3_usize));
    assert_eq!(v.xyww(), usizevec4(1_usize, 2_usize, 4_usize, 4_usize));
    assert_eq!(v.xzxx(), usizevec4(1_usize, 3_usize, 1_usize, 1_usize));
    assert_eq!(v.xzxy(), usizevec4(1_usize, 3_usize, 1_usize, 2_usize));
    assert_eq!(v.xzxz(), usizevec4(1_usize, 3_usize, 1_usize, 3_usize));
    assert_eq!(v.xzxw(), usizevec4(1_usize, 3_usize, 1_usize, 4_usize));
    assert_eq!(v.xzyx(), usizevec4(1_usize, 3_usize, 2_usize, 1_usize));
    assert_eq!(v.xzyy(), usizevec4(1_usize, 3_usize, 2_usize, 2_usize));
    assert_eq!(v.xzyz(), usizevec4(1_usize, 3_usize, 2_usize, 3_usize));
    assert_eq!(v.xzyw(), usizevec4(1_usize, 3_usize, 2_usize, 4_usize));
    assert_eq!(v.xzzx(), usizevec4(1_usize, 3_usize, 3_usize, 1_usize));
    assert_eq!(v.xzzy(), usizevec4(1_usize, 3_usize, 3_usize, 2_usize));
    assert_eq!(v.xzzz(), usizevec4(1_usize, 3_usize, 3_usize, 3_usize));
    assert_eq!(v.xzzw(), usizevec4(1_usize, 3_usize, 3_usize, 4_usize));
    assert_eq!(v.xzwx(), usizevec4(1_usize, 3_usize, 4_usize, 1_usize));
    assert_eq!(v.xzwy(), usizevec4(1_usize, 3_usize, 4_usize, 2_usize));
    assert_eq!(v.xzwz(), usizevec4(1_usize, 3_usize, 4_usize, 3_usize));
    assert_eq!(v.xzww(), usizevec4(1_usize, 3_usize, 4_usize, 4_usize));
    assert_eq!(v.xwxx(), usizevec4(1_usize, 4_usize, 1_usize, 1_usize));
    assert_eq!(v.xwxy(), usizevec4(1_usize, 4_usize, 1_usize, 2_usize));
    assert_eq!(v.xwxz(), usizevec4(1_usize, 4_usize, 1_usize, 3_usize));
    assert_eq!(v.xwxw(), usizevec4(1_usize, 4_usize, 1_usize, 4_usize));
    assert_eq!(v.xwyx(), usizevec4(1_usize, 4_usize, 2_usize, 1_usize));
    assert_eq!(v.xwyy(), usizevec4(1_usize, 4_usize, 2_usize, 2_usize));
    assert_eq!(v.xwyz(), usizevec4(1_usize, 4_usize, 2_usize, 3_usize));
    assert_eq!(v.xwyw(), usizevec4(1_usize, 4_usize, 2_usize, 4_usize));
    assert_eq!(v.xwzx(), usizevec4(1_usize, 4_usize, 3_usize, 1_usize));
    assert_eq!(v.xwzy(), usizevec4(1_usize, 4_usize, 3_usize, 2_usize));
    assert_eq!(v.xwzz(), usizevec4(1_usize, 4_usize, 3_usize, 3_usize));
    assert_eq!(v.xwzw(), usizevec4(1_usize, 4_usize, 3_usize, 4_usize));
    assert_eq!(v.xwwx(), usizevec4(1_usize, 4_usize, 4_usize, 1_usize));
    assert_eq!(v.xwwy(), usizevec4(1_usize, 4_usize, 4_usize, 2_usize));
    assert_eq!(v.xwwz(), usizevec4(1_usize, 4_usize, 4_usize, 3_usize));
    assert_eq!(v.xwww(), usizevec4(1_usize, 4_usize, 4_usize, 4_usize));
    assert_eq!(v.yxxx(), usizevec4(2_usize, 1_usize, 1_usize, 1_usize));
    assert_eq!(v.yxxy(), usizevec4(2_usize, 1_usize, 1_usize, 2_usize));
    assert_eq!(v.yxxz(), usizevec4(2_usize, 1_usize, 1_usize, 3_usize));
    assert_eq!(v.yxxw(), usizevec4(2_usize, 1_usize, 1_usize, 4_usize));
    assert_eq!(v.yxyx(), usizevec4(2_usize, 1_usize, 2_usize, 1_usize));
    assert_eq!(v.yxyy(), usizevec4(2_usize, 1_usize, 2_usize, 2_usize));
    assert_eq!(v.yxyz(), usizevec4(2_usize, 1_usize, 2_usize, 3_usize));
    assert_eq!(v.yxyw(), usizevec4(2_usize, 1_usize, 2_usize, 4_usize));
    assert_eq!(v.yxzx(), usizevec4(2_usize, 1_usize, 3_usize, 1_usize));
    assert_eq!(v.yxzy(), usizevec4(2_usize, 1_usize, 3_usize, 2_usize));
    assert_eq!(v.yxzz(), usizevec4(2_usize, 1_usize, 3_usize, 3_usize));
    assert_eq!(v.yxzw(), usizevec4(2_usize, 1_usize, 3_usize, 4_usize));
    assert_eq!(v.yxwx(), usizevec4(2_usize, 1_usize, 4_usize, 1_usize));
    assert_eq!(v.yxwy(), usizevec4(2_usize, 1_usize, 4_usize, 2_usize));
    assert_eq!(v.yxwz(), usizevec4(2_usize, 1_usize, 4_usize, 3_usize));
    assert_eq!(v.yxww(), usizevec4(2_usize, 1_usize, 4_usize, 4_usize));
    assert_eq!(v.yyxx(), usizevec4(2_usize, 2_usize, 1_usize, 1_usize));
    assert_eq!(v.yyxy(), usizevec4(2_usize, 2_usize, 1_usize, 2_usize));
    assert_eq!(v.yyxz(), usizevec4(2_usize, 2_usize, 1_usize, 3_usize));
    assert_eq!(v.yyxw(), usizevec4(2_usize, 2_usize, 1_usize, 4_usize));
    assert_eq!(v.yyyx(), usizevec4(2_usize, 2_usize, 2_usize, 1_usize));
    assert_eq!(v.yyyy(), usizevec4(2_usize, 2_usize, 2_usize, 2_usize));
    assert_eq!(v.yyyz(), usizevec4(2_usize, 2_usize, 2_usize, 3_usize));
    assert_eq!(v.yyyw(), usizevec4(2_usize, 2_usize, 2_usize, 4_usize));
    assert_eq!(v.yyzx(), usizevec4(2_usize, 2_usize, 3_usize, 1_usize));
    assert_eq!(v.yyzy(), usizevec4(2_usize, 2_usize, 3_usize, 2_usize));
    assert_eq!(v.yyzz(), usizevec4(2_usize, 2_usize, 3_usize, 3_usize));
    assert_eq!(v.yyzw(), usizevec4(2_usize, 2_usize, 3_usize, 4_usize));
    assert_eq!(v.yywx(), usizevec4(2_usize, 2_usize, 4_usize, 1_usize));
    assert_eq!(v.yywy(), usizevec4(2_usize, 2_usize, 4_usize, 2_usize));
    assert_eq!(v.yywz(), usizevec4(2_usize, 2_usize, 4_usize, 3_usize));
    assert_eq!(v.yyww(), usizevec4(2_usize, 2_usize, 4_usize, 4_usize));
    assert_eq!(v.yzxx(), usizevec4(2_usize, 3_usize, 1_usize, 1_usize));
    assert_eq!(v.yzxy(), usizevec4(2_usize, 3_usize, 1_usize, 2_usize));
    assert_eq!(v.yzxz(), usizevec4(2_usize, 3_usize, 1_usize, 3_usize));
    assert_eq!(v.yzxw(), usizevec4(2_usize, 3_usize, 1_usize, 4_usize));
    assert_eq!(v.yzyx(), usizevec4(2_usize, 3_usize, 2_usize, 1_usize));
    assert_eq!(v.yzyy(), usizevec4(2_usize, 3_usize, 2_usize, 2_usize));
    assert_eq!(v.yzyz(), usizevec4(2_usize, 3_usize, 2_usize, 3_usize));
    assert_eq!(v.yzyw(), usizevec4(2_usize, 3_usize, 2_usize, 4_usize));
    assert_eq!(v.yzzx(), usizevec4(2_usize, 3_usize, 3_usize, 1_usize));
    assert_eq!(v.yzzy(), usizevec4(2_usize, 3_usize, 3_usize, 2_usize));
    assert_eq!(v.yzzz(), usizevec4(2_usize, 3_usize, 3_usize, 3_usize));
    assert_eq!(v.yzzw(), usizevec4(2_usize, 3_usize, 3_usize, 4_usize));
    assert_eq!(v.yzwx(), usizevec4(2_usize, 3_usize, 4_usize, 1_usize));
    assert_eq!(v.yzwy(), usizevec4(2_usize, 3_usize, 4_usize, 2_usize));
    assert_eq!(v.yzwz(), usizevec4(2_usize, 3_usize, 4_usize, 3_usize));
    assert_eq!(v.yzww(), usizevec4(2_usize, 3_usize, 4_usize, 4_usize));
    assert_eq!(v.ywxx(), usizevec4(2_usize, 4_usize, 1_usize, 1_usize));
    assert_eq!(v.ywxy(), usizevec4(2_usize, 4_usize, 1_usize, 2_usize));
    assert_eq!(v.ywxz(), usizevec4(2_usize, 4_usize, 1_usize, 3_usize));
    assert_eq!(v.ywxw(), usizevec4(2_usize, 4_usize, 1_usize, 4_usize));
    assert_eq!(v.ywyx(), usizevec4(2_usize, 4_usize, 2_usize, 1_usize));
    assert_eq!(v.ywyy(), usizevec4(2_usize, 4_usize, 2_usize, 2_usize));
    assert_eq!(v.ywyz(), usizevec4(2_usize, 4_usize, 2_usize, 3_usize));
    assert_eq!(v.ywyw(), usizevec4(2_usize, 4_usize, 2_usize, 4_usize));
    assert_eq!(v.ywzx(), usizevec4(2_usize, 4_usize, 3_usize, 1_usize));
    assert_eq!(v.ywzy(), usizevec4(2_usize, 4_usize, 3_usize, 2_usize));
    assert_eq!(v.ywzz(), usizevec4(2_usize, 4_usize, 3_usize, 3_usize));
    assert_eq!(v.ywzw(), usizevec4(2_usize, 4_usize, 3_usize, 4_usize));
    assert_eq!(v.ywwx(), usizevec4(2_usize, 4_usize, 4_usize, 1_usize));
    assert_eq!(v.ywwy(), usizevec4(2_usize, 4_usize, 4_usize, 2_usize));
    assert_eq!(v.ywwz(), usizevec4(2_usize, 4_usize, 4_usize, 3_usize));
    assert_eq!(v.ywww(), usizevec4(2_usize, 4_usize, 4_usize, 4_usize));
    assert_eq!(v.zxxx(), usizevec4(3_usize, 1_usize, 1_usize, 1_usize));
    assert_eq!(v.zxxy(), usizevec4(3_usize, 1_usize, 1_usize, 2_usize));
    assert_eq!(v.zxxz(), usizevec4(3_usize, 1_usize, 1_usize, 3_usize));
    assert_eq!(v.zxxw(), usizevec4(3_usize, 1_usize, 1_usize, 4_usize));
    assert_eq!(v.zxyx(), usizevec4(3_usize, 1_usize, 2_usize, 1_usize));
    assert_eq!(v.zxyy(), usizevec4(3_usize, 1_usize, 2_usize, 2_usize));
    assert_eq!(v.zxyz(), usizevec4(3_usize, 1_usize, 2_usize, 3_usize));
    assert_eq!(v.zxyw(), usizevec4(3_usize, 1_usize, 2_usize, 4_usize));
    assert_eq!(v.zxzx(), usizevec4(3_usize, 1_usize, 3_usize, 1_usize));
    assert_eq!(v.zxzy(), usizevec4(3_usize, 1_usize, 3_usize, 2_usize));
    assert_eq!(v.zxzz(), usizevec4(3_usize, 1_usize, 3_usize, 3_usize));
    assert_eq!(v.zxzw(), usizevec4(3_usize, 1_usize, 3_usize, 4_usize));
    assert_eq!(v.zxwx(), usizevec4(3_usize, 1_usize, 4_usize, 1_usize));
    assert_eq!(v.zxwy(), usizevec4(3_usize, 1_usize, 4_usize, 2_usize));
    assert_eq!(v.zxwz(), usizevec4(3_usize, 1_usize, 4_usize, 3_usize));
    assert_eq!(v.zxww(), usizevec4(3_usize, 1_usize, 4_usize, 4_usize));
    assert_eq!(v.zyxx(), usizevec4(3_usize, 2_usize, 1_usize, 1_usize));
    assert_eq!(v.zyxy(), usizevec4(3_usize, 2_usize, 1_usize, 2_usize));
    assert_eq!(v.zyxz(), usizevec4(3_usize, 2_usize, 1_usize, 3_usize));
    assert_eq!(v.zyxw(), usizevec4(3_usize, 2_usize, 1_usize, 4_usize));
    assert_eq!(v.zyyx(), usizevec4(3_usize, 2_usize, 2_usize, 1_usize));
    assert_eq!(v.zyyy(), usizevec4(3_usize, 2_usize, 2_usize, 2_usize));
    assert_eq!(v.zyyz(), usizevec4(3_usize, 2_usize, 2_usize, 3_usize));
    assert_eq!(v.zyyw(), usizevec4(3_usize, 2_usize, 2_usize, 4_usize));
    assert_eq!(v.zyzx(), usizevec4(3_usize, 2_usize, 3_usize, 1_usize));
    assert_eq!(v.zyzy(), usizevec4(3_usize, 2_usize, 3_usize, 2_usize));
    assert_eq!(v.zyzz(), usizevec4(3_usize, 2_usize, 3_usize, 3_usize));
    assert_eq!(v.zyzw(), usizevec4(3_usize, 2_usize, 3_usize, 4_usize));
    assert_eq!(v.zywx(), usizevec4(3_usize, 2_usize, 4_usize, 1_usize));
    assert_eq!(v.zywy(), usizevec4(3_usize, 2_usize, 4_usize, 2_usize));
    assert_eq!(v.zywz(), usizevec4(3_usize, 2_usize, 4_usize, 3_usize));
    assert_eq!(v.zyww(), usizevec4(3_usize, 2_usize, 4_usize, 4_usize));
    assert_eq!(v.zzxx(), usizevec4(3_usize, 3_usize, 1_usize, 1_usize));
    assert_eq!(v.zzxy(), usizevec4(3_usize, 3_usize, 1_usize, 2_usize));
    assert_eq!(v.zzxz(), usizevec4(3_usize, 3_usize, 1_usize, 3_usize));
    assert_eq!(v.zzxw(), usizevec4(3_usize, 3_usize, 1_usize, 4_usize));
    assert_eq!(v.zzyx(), usizevec4(3_usize, 3_usize, 2_usize, 1_usize));
    assert_eq!(v.zzyy(), usizevec4(3_usize, 3_usize, 2_usize, 2_usize));
    assert_eq!(v.zzyz(), usizevec4(3_usize, 3_usize, 2_usize, 3_usize));
    assert_eq!(v.zzyw(), usizevec4(3_usize, 3_usize, 2_usize, 4_usize));
    assert_eq!(v.zzzx(), usizevec4(3_usize, 3_usize, 3_usize, 1_usize));
    assert_eq!(v.zzzy(), usizevec4(3_usize, 3_usize, 3_usize, 2_usize));
    assert_eq!(v.zzzz(), usizevec4(3_usize, 3_usize, 3_usize, 3_usize));
    assert_eq!(v.zzzw(), usizevec4(3_usize, 3_usize, 3_usize, 4_usize));
    assert_eq!(v.zzwx(), usizevec4(3_usize, 3_usize, 4_usize, 1_usize));
    assert_eq!(v.zzwy(), usizevec4(3_usize, 3_usize, 4_usize, 2_usize));
    assert_eq!(v.zzwz(), usizevec4(3_usize, 3_usize, 4_usize, 3_usize));
    assert_eq!(v.zzww(), usizevec4(3_usize, 3_usize, 4_usize, 4_usize));
    assert_eq!(v.zwxx(), usizevec4(3_usize, 4_usize, 1_usize, 1_usize));
    assert_eq!(v.zwxy(), usizevec4(3_usize, 4_usize, 1_usize, 2_usize));
    assert_eq!(v.zwxz(), usizevec4(3_usize, 4_usize, 1_usize, 3_usize));
    assert_eq!(v.zwxw(), usizevec4(3_usize, 4_usize, 1_usize, 4_usize));
    assert_eq!(v.zwyx(), usizevec4(3_usize, 4_usize, 2_usize, 1_usize));
    assert_eq!(v.zwyy(), usizevec4(3_usize, 4_usize, 2_usize, 2_usize));
    assert_eq!(v.zwyz(), usizevec4(3_usize, 4_usize, 2_usize, 3_usize));
    assert_eq!(v.zwyw(), usizevec4(3_usize, 4_usize, 2_usize, 4_usize));
    assert_eq!(v.zwzx(), usizevec4(3_usize, 4_usize, 3_usize, 1_usize));
    assert_eq!(v.zwzy(), usizevec4(3_usize, 4_usize, 3_usize, 2_usize));
    assert_eq!(v.zwzz(), usizevec4(3_usize, 4_usize, 3_usize, 3_usize));
    assert_eq!(v.zwzw(), usizevec4(3_usize, 4_usize, 3_usize, 4_usize));
    assert_eq!(v.zwwx(), usizevec4(3_usize, 4_usize, 4_usize, 1_usize));
    assert_eq!(v.zwwy(), usizevec4(3_usize, 4_usize, 4_usize, 2_usize));
    assert_eq!(v.zwwz(), usizevec4(3_usize, 4_usize, 4_usize, 3_usize));
    assert_eq!(v.zwww(), usizevec4(3_usize, 4_usize, 4_usize, 4_usize));
    assert_eq!(v.wxxx(), usizevec4(4_usize, 1_usize, 1_usize, 1_usize));
    assert_eq!(v.wxxy(), usizevec4(4_usize, 1_usize, 1_usize, 2_usize));
    assert_eq!(v.wxxz(), usizevec4(4_usize, 1_usize, 1_usize, 3_usize));
    assert_eq!(v.wxxw(), usizevec4(4_usize, 1_usize, 1_usize, 4_usize));
    assert_eq!(v.wxyx(), usizevec4(4_usize, 1_usize, 2_usize, 1_usize));
    assert_eq!(v.wxyy(), usizevec4(4_usize, 1_usize, 2_usize, 2_usize));
    assert_eq!(v.wxyz(), usizevec4(4_usize, 1_usize, 2_usize, 3_usize));
    assert_eq!(v.wxyw(), usizevec4(4_usize, 1_usize, 2_usize, 4_usize));
    assert_eq!(v.wxzx(), usizevec4(4_usize, 1_usize, 3_usize, 1_usize));
    assert_eq!(v.wxzy(), usizevec4(4_usize, 1_usize, 3_usize, 2_usize));
    assert_eq!(v.wxzz(), usizevec4(4_usize, 1_usize, 3_usize, 3_usize));
    assert_eq!(v.wxzw(), usizevec4(4_usize, 1_usize, 3_usize, 4_usize));
    assert_eq!(v.wxwx(), usizevec4(4_usize, 1_usize, 4_usize, 1_usize));
    assert_eq!(v.wxwy(), usizevec4(4_usize, 1_usize, 4_usize, 2_usize));
    assert_eq!(v.wxwz(), usizevec4(4_usize, 1_usize, 4_usize, 3_usize));
    assert_eq!(v.wxww(), usizevec4(4_usize, 1_usize, 4_usize, 4_usize));
    assert_eq!(v.wyxx(), usizevec4(4_usize, 2_usize, 1_usize, 1_usize));
    assert_eq!(v.wyxy(), usizevec4(4_usize, 2_usize, 1_usize, 2_usize));
    assert_eq!(v.wyxz(), usizevec4(4_usize, 2_usize, 1_usize, 3_usize));
    assert_eq!(v.wyxw(), usizevec4(4_usize, 2_usize, 1_usize, 4_usize));
    assert_eq!(v.wyyx(), usizevec4(4_usize, 2_usize, 2_usize, 1_usize));
    assert_eq!(v.wyyy(), usizevec4(4_usize, 2_usize, 2_usize, 2_usize));
    assert_eq!(v.wyyz(), usizevec4(4_usize, 2_usize, 2_usize, 3_usize));
    assert_eq!(v.wyyw(), usizevec4(4_usize, 2_usize, 2_usize, 4_usize));
    assert_eq!(v.wyzx(), usizevec4(4_usize, 2_usize, 3_usize, 1_usize));
    assert_eq!(v.wyzy(), usizevec4(4_usize, 2_usize, 3_usize, 2_usize));
    assert_eq!(v.wyzz(), usizevec4(4_usize, 2_usize, 3_usize, 3_usize));
    assert_eq!(v.wyzw(), usizevec4(4_usize, 2_usize, 3_usize, 4_usize));
    assert_eq!(v.wywx(), usizevec4(4_usize, 2_usize, 4_usize, 1_usize));
    assert_eq!(v.wywy(), usizevec4(4_usize, 2_usize, 4_usize, 2_usize));
    assert_eq!(v.wywz(), usizevec4(4_usize, 2_usize, 4_usize, 3_usize));
    assert_eq!(v.wyww(), usizevec4(4_usize, 2_usize, 4_usize, 4_usize));
    assert_eq!(v.wzxx(), usizevec4(4_usize, 3_usize, 1_usize, 1_usize));
    assert_eq!(v.wzxy(), usizevec4(4_usize, 3_usize, 1_usize, 2_usize));
    assert_eq!(v.wzxz(), usizevec4(4_usize, 3_usize, 1_usize, 3_usize));
    assert_eq!(v.wzxw(), usizevec4(4_usize, 3_usize, 1_usize, 4_usize));
    assert_eq!(v.wzyx(), usizevec4(4_usize, 3_usize, 2_usize, 1_usize));
    assert_eq!(v.wzyy(), usizevec4(4_usize, 3_usize, 2_usize, 2_usize));
    assert_eq!(v.wzyz(), usizevec4(4_usize, 3_usize, 2_usize, 3_usize));
    assert_eq!(v.wzyw(), usizevec4(4_usize, 3_usize, 2_usize, 4_usize));
    assert_eq!(v.wzzx(), usizevec4(4_usize, 3_usize, 3_usize, 1_usize));
    assert_eq!(v.wzzy(), usizevec4(4_usize, 3_usize, 3_usize, 2_usize));
    assert_eq!(v.wzzz(), usizevec4(4_usize, 3_usize, 3_usize, 3_usize));
    assert_eq!(v.wzzw(), usizevec4(4_usize, 3_usize, 3_usize, 4_usize));
    assert_eq!(v.wzwx(), usizevec4(4_usize, 3_usize, 4_usize, 1_usize));
    assert_eq!(v.wzwy(), usizevec4(4_usize, 3_usize, 4_usize, 2_usize));
    assert_eq!(v.wzwz(), usizevec4(4_usize, 3_usize, 4_usize, 3_usize));
    assert_eq!(v.wzww(), usizevec4(4_usize, 3_usize, 4_usize, 4_usize));
    assert_eq!(v.wwxx(), usizevec4(4_usize, 4_usize, 1_usize, 1_usize));
    assert_eq!(v.wwxy(), usizevec4(4_usize, 4_usize, 1_usize, 2_usize));
    assert_eq!(v.wwxz(), usizevec4(4_usize, 4_usize, 1_usize, 3_usize));
    assert_eq!(v.wwxw(), usizevec4(4_usize, 4_usize, 1_usize, 4_usize));
    assert_eq!(v.wwyx(), usizevec4(4_usize, 4_usize, 2_usize, 1_usize));
    assert_eq!(v.wwyy(), usizevec4(4_usize, 4_usize, 2_usize, 2_usize));
    assert_eq!(v.wwyz(), usizevec4(4_usize, 4_usize, 2_usize, 3_usize));
    assert_eq!(v.wwyw(), usizevec4(4_usize, 4_usize, 2_usize, 4_usize));
    assert_eq!(v.wwzx(), usizevec4(4_usize, 4_usize, 3_usize, 1_usize));
    assert_eq!(v.wwzy(), usizevec4(4_usize, 4_usize, 3_usize, 2_usize));
    assert_eq!(v.wwzz(), usizevec4(4_usize, 4_usize, 3_usize, 3_usize));
    assert_eq!(v.wwzw(), usizevec4(4_usize, 4_usize, 3_usize, 4_usize));
    assert_eq!(v.wwwx(), usizevec4(4_usize, 4_usize, 4_usize, 1_usize));
    assert_eq!(v.wwwy(), usizevec4(4_usize, 4_usize, 4_usize, 2_usize));
    assert_eq!(v.wwwz(), usizevec4(4_usize, 4_usize, 4_usize, 3_usize));
    assert_eq!(v.wwww(), usizevec4(4_usize, 4_usize, 4_usize, 4_usize));
    assert_eq!(v.xxx(), usizevec3(1_usize, 1_usize, 1_usize));
    assert_eq!(v.xxy(), usizevec3(1_usize, 1_usize, 2_usize));
    assert_eq!(v.xxz(), usizevec3(1_usize, 1_usize, 3_usize));
    assert_eq!(v.xxw(), usizevec3(1_usize, 1_usize, 4_usize));
    assert_eq!(v.xyx(), usizevec3(1_usize, 2_usize, 1_usize));
    assert_eq!(v.xyy(), usizevec3(1_usize, 2_usize, 2_usize));
    assert_eq!(v.xyz(), usizevec3(1_usize, 2_usize, 3_usize));
    assert_eq!(v.xyw(), usizevec3(1_usize, 2_usize, 4_usize));
    assert_eq!(v.xzx(), usizevec3(1_usize, 3_usize, 1_usize));
    assert_eq!(v.xzy(), usizevec3(1_usize, 3_usize, 2_usize));
    assert_eq!(v.xzz(), usizevec3(1_usize, 3_usize, 3_usize));
    assert_eq!(v.xzw(), usizevec3(1_usize, 3_usize, 4_usize));
    assert_eq!(v.xwx(), usizevec3(1_usize, 4_usize, 1_usize));
    assert_eq!(v.xwy(), usizevec3(1_usize, 4_usize, 2_usize));
    assert_eq!(v.xwz(), usizevec3(1_usize, 4_usize, 3_usize));
    assert_eq!(v.xww(), usizevec3(1_usize, 4_usize, 4_usize));
    assert_eq!(v.yxx(), usizevec3(2_usize, 1_usize, 1_usize));
    assert_eq!(v.yxy(), usizevec3(2_usize, 1_usize, 2_usize));
    assert_eq!(v.yxz(), usizevec3(2_usize, 1_usize, 3_usize));
    assert_eq!(v.yxw(), usizevec3(2_usize, 1_usize, 4_usize));
    assert_eq!(v.yyx(), usizevec3(2_usize, 2_usize, 1_usize));
    assert_eq!(v.yyy(), usizevec3(2_usize, 2_usize, 2_usize));
    assert_eq!(v.yyz(), usizevec3(2_usize, 2_usize, 3_usize));
    assert_eq!(v.yyw(), usizevec3(2_usize, 2_usize, 4_usize));
    assert_eq!(v.yzx(), usizevec3(2_usize, 3_usize, 1_usize));
    assert_eq!(v.yzy(), usizevec3(2_usize, 3_usize, 2_usize));
    assert_eq!(v.yzz(), usizevec3(2_usize, 3_usize, 3_usize));
    assert_eq!(v.yzw(), usizevec3(2_usize, 3_usize, 4_usize));
    assert_eq!(v.ywx(), usizevec3(2_usize, 4_usize, 1_usize));
    assert_eq!(v.ywy(), usizevec3(2_usize, 4_usize, 2_usize));
    assert_eq!(v.ywz(), usizevec3(2_usize, 4_usize, 3_usize));
    assert_eq!(v.yww(), usizevec3(2_usize, 4_usize, 4_usize));
    assert_eq!(v.zxx(), usizevec3(3_usize, 1_usize, 1_usize));
    assert_eq!(v.zxy(), usizevec3(3_usize, 1_usize, 2_usize));
    assert_eq!(v.zxz(), usizevec3(3_usize, 1_usize, 3_usize));
    assert_eq!(v.zxw(), usizevec3(3_usize, 1_usize, 4_usize));
    assert_eq!(v.zyx(), usizevec3(3_usize, 2_usize, 1_usize));
    assert_eq!(v.zyy(), usizevec3(3_usize, 2_usize, 2_usize));
    assert_eq!(v.zyz(), usizevec3(3_usize, 2_usize, 3_usize));
    assert_eq!(v.zyw(), usizevec3(3_usize, 2_usize, 4_usize));
    assert_eq!(v.zzx(), usizevec3(3_usize, 3_usize, 1_usize));
    assert_eq!(v.zzy(), usizevec3(3_usize, 3_usize, 2_usize));
    assert_eq!(v.zzz(), usizevec3(3_usize, 3_usize, 3_usize));
    assert_eq!(v.zzw(), usizevec3(3_usize, 3_usize, 4_usize));
    assert_eq!(v.zwx(), usizevec3(3_usize, 4_usize, 1_usize));
    assert_eq!(v.zwy(), usizevec3(3_usize, 4_usize, 2_usize));
    assert_eq!(v.zwz(), usizevec3(3_usize, 4_usize, 3_usize));
    assert_eq!(v.zww(), usizevec3(3_usize, 4_usize, 4_usize));
    assert_eq!(v.wxx(), usizevec3(4_usize, 1_usize, 1_usize));
    assert_eq!(v.wxy(), usizevec3(4_usize, 1_usize, 2_usize));
    assert_eq!(v.wxz(), usizevec3(4_usize, 1_usize, 3_usize));
    assert_eq!(v.wxw(), usizevec3(4_usize, 1_usize, 4_usize));
    assert_eq!(v.wyx(), usizevec3(4_usize, 2_usize, 1_usize));
    assert_eq!(v.wyy(), usizevec3(4_usize, 2_usize, 2_usize));
    assert_eq!(v.wyz(), usizevec3(4_usize, 2_usize, 3_usize));
    assert_eq!(v.wyw(), usizevec3(4_usize, 2_usize, 4_usize));
    assert_eq!(v.wzx(), usizevec3(4_usize, 3_usize, 1_usize));
    assert_eq!(v.wzy(), usizevec3(4_usize, 3_usize, 2_usize));
    assert_eq!(v.wzz(), usizevec3(4_usize, 3_usize, 3_usize));
    assert_eq!(v.wzw(), usizevec3(4_usize, 3_usize, 4_usize));
    assert_eq!(v.wwx(), usizevec3(4_usize, 4_usize, 1_usize));
    assert_eq!(v.wwy(), usizevec3(4_usize, 4_usize, 2_usize));
    assert_eq!(v.wwz(), usizevec3(4_usize, 4_usize, 3_usize));
    assert_eq!(v.www(), usizevec3(4_usize, 4_usize, 4_usize));
    assert_eq!(
        v.with_xyz(rhs3),
        usizevec4(11_usize, 12_usize, 13_usize, 4_usize)
    );
    assert_eq!(
        v.with_xyw(rhs3),
        usizevec4(11_usize, 12_usize, 3_usize, 13_usize)
    );
    assert_eq!(
        v.with_xzy(rhs3),
        usizevec4(11_usize, 13_usize, 12_usize, 4_usize)
    );
    assert_eq!(
        v.with_xzw(rhs3),
        usizevec4(11_usize, 2_usize, 12_usize, 13_usize)
    );
    assert_eq!(
        v.with_xwy(rhs3),
        usizevec4(11_usize, 13_usize, 3_usize, 12_usize)
    );
    assert_eq!(
        v.with_xwz(rhs3),
        usizevec4(11_usize, 2_usize, 13_usize, 12_usize)
    );
    assert_eq!(
        v.with_yxz(rhs3),
        usizevec4(12_usize, 11_usize, 13_usize, 4_usize)
    );
    assert_eq!(
        v.with_yxw(rhs3),
        usizevec4(12_usize, 11_usize, 3_usize, 13_usize)
    );
    assert_eq!(
        v.with_yzx(rhs3),
        usizevec4(13_usize, 11_usize, 12_usize, 4_usize)
    );
    assert_eq!(
        v.with_yzw(rhs3),
        usizevec4(1_usize, 11_usize, 12_usize, 13_usize)
    );
    assert_eq!(
        v.with_ywx(rhs3),
        usizevec4(13_usize, 11_usize, 3_usize, 12_usize)
    );
    assert_eq!(
        v.with_ywz(rhs3),
        usizevec4(1_usize, 11_usize, 13_usize, 12_usize)
    );
    assert_eq!(
        v.with_zxy(rhs3),
        usizevec4(12_usize, 13_usize, 11_usize, 4_usize)
    );
    assert_eq!(
        v.with_zxw(rhs3),
        usizevec4(12_usize, 2_usize, 11_usize, 13_usize)
    );
    assert_eq!(
        v.with_zyx(rhs3),
        usizevec4(13_usize, 12_usize, 11_usize, 4_usize)
    );
    assert_eq!(
        v.with_zyw(rhs3),
        usizevec4(1_usize, 12_usize, 11_usize, 13_usize)
    );
    assert_eq!(
        v.with_zwx(rhs3),
        usizevec4(13_usize, 2_usize, 11_usize, 12_usize)
    );
    assert_eq!(
        v.with_zwy(rhs3),
        usizevec4(1_usize, 13_usize, 11_usize, 12_usize)
    );
    assert_eq!(
        v.with_wxy(rhs3),
        usizevec4(12_usize, 13_usize, 3_usize, 11_usize)
    );
    assert_eq!(
        v.with_wxz(rhs3),
        usizevec4(12_usize, 2_usize, 13_usize, 11_usize)
    );
    assert_eq!(
        v.with_wyx(rhs3),
        usizevec4(13_usize, 12_usize, 3_usize, 11_usize)
    );
    assert_eq!(
        v.with_wyz(rhs3),
        usizevec4(1_usize, 12_usize, 13_usize, 11_usize)
    );
    assert_eq!(
        v.with_wzx(rhs3),
        usizevec4(13_usize, 2_usize, 12_usize, 11_usize)
    );
    assert_eq!(
        v.with_wzy(rhs3),
        usizevec4(1_usize, 13_usize, 12_usize, 11_usize)
    );
    assert_eq!(v.xx(), usizevec2(1_usize, 1_usize));
    assert_eq!(v.xy(), usizevec2(1_usize, 2_usize));
    assert_eq!(v.xz(), usizevec2(1_usize, 3_usize));
    assert_eq!(v.xw(), usizevec2(1_usize, 4_usize));
    assert_eq!(v.yx(), usizevec2(2_usize, 1_usize));
    assert_eq!(v.yy(), usizevec2(2_usize, 2_usize));
    assert_eq!(v.yz(), usizevec2(2_usize, 3_usize));
    assert_eq!(v.yw(), usizevec2(2_usize, 4_usize));
    assert_eq!(v.zx(), usizevec2(3_usize, 1_usize));
    assert_eq!(v.zy(), usizevec2(3_usize, 2_usize));
    assert_eq!(v.zz(), usizevec2(3_usize, 3_usize));
    assert_eq!(v.zw(), usizevec2(3_usize, 4_usize));
    assert_eq!(v.wx(), usizevec2(4_usize, 1_usize));
    assert_eq!(v.wy(), usizevec2(4_usize, 2_usize));
    assert_eq!(v.wz(), usizevec2(4_usize, 3_usize));
    assert_eq!(v.ww(), usizevec2(4_usize, 4_usize));
    assert_eq!(
        v.with_xy(rhs2),
        usizevec4(11_usize, 12_usize, 3_usize, 4_usize)
    );
    assert_eq!(
        v.with_xz(rhs2),
        usizevec4(11_usize, 2_usize, 12_usize, 4_usize)
    );
    assert_eq!(
        v.with_xw(rhs2),
        usizevec4(11_usize, 2_usize, 3_usize, 12_usize)
    );
    assert_eq!(
        v.with_yx(rhs2),
        usizevec4(12_usize, 11_usize, 3_usize, 4_usize)
    );
    assert_eq!(
        v.with_yz(rhs2),
        usizevec4(1_usize, 11_usize, 12_usize, 4_usize)
    );
    assert_eq!(
        v.with_yw(rhs2),
        usizevec4(1_usize, 11_usize, 3_usize, 12_usize)
    );
    assert_eq!(
        v.with_zx(rhs2),
        usizevec4(12_usize, 2_usize, 11_usize, 4_usize)
    );
    assert_eq!(
        v.with_zy(rhs2),
        usizevec4(1_usize, 12_usize, 11_usize, 4_usize)
    );
    assert_eq!(
        v.with_zw(rhs2),
        usizevec4(1_usize, 2_usize, 11_usize, 12_usize)
    );
    assert_eq!(
        v.with_wx(rhs2),
        usizevec4(12_usize, 2_usize, 3_usize, 11_usize)
    );
    assert_eq!(
        v.with_wy(rhs2),
        usizevec4(1_usize, 12_usize, 3_usize, 11_usize)
    );
    assert_eq!(
        v.with_wz(rhs2),
        usizevec4(1_usize, 2_usize, 12_usize, 11_usize)
    );
});

glam_test!(test_usizevec3_swizzles, {
    let v = usizevec3(1_usize, 2_usize, 3_usize);
    let rhs2 = usizevec2(11_usize, 12_usize);
    assert_eq!(v, v.xyz());
    assert_eq!(v.xxxx(), usizevec4(1_usize, 1_usize, 1_usize, 1_usize));
    assert_eq!(v.xxxy(), usizevec4(1_usize, 1_usize, 1_usize, 2_usize));
    assert_eq!(v.xxxz(), usizevec4(1_usize, 1_usize, 1_usize, 3_usize));
    assert_eq!(v.xxyx(), usizevec4(1_usize, 1_usize, 2_usize, 1_usize));
    assert_eq!(v.xxyy(), usizevec4(1_usize, 1_usize, 2_usize, 2_usize));
    assert_eq!(v.xxyz(), usizevec4(1_usize, 1_usize, 2_usize, 3_usize));
    assert_eq!(v.xxzx(), usizevec4(1_usize, 1_usize, 3_usize, 1_usize));
    assert_eq!(v.xxzy(), usizevec4(1_usize, 1_usize, 3_usize, 2_usize));
    assert_eq!(v.xxzz(), usizevec4(1_usize, 1_usize, 3_usize, 3_usize));
    assert_eq!(v.xyxx(), usizevec4(1_usize, 2_usize, 1_usize, 1_usize));
    assert_eq!(v.xyxy(), usizevec4(1_usize, 2_usize, 1_usize, 2_usize));
    assert_eq!(v.xyxz(), usizevec4(1_usize, 2_usize, 1_usize, 3_usize));
    assert_eq!(v.xyyx(), usizevec4(1_usize, 2_usize, 2_usize, 1_usize));
    assert_eq!(v.xyyy(), usizevec4(1_usize, 2_usize, 2_usize, 2_usize));
    assert_eq!(v.xyyz(), usizevec4(1_usize, 2_usize, 2_usize, 3_usize));
    assert_eq!(v.xyzx(), usizevec4(1_usize, 2_usize, 3_usize, 1_usize));
    assert_eq!(v.xyzy(), usizevec4(1_usize, 2_usize, 3_usize, 2_usize));
    assert_eq!(v.xyzz(), usizevec4(1_usize, 2_usize, 3_usize, 3_usize));
    assert_eq!(v.xzxx(), usizevec4(1_usize, 3_usize, 1_usize, 1_usize));
    assert_eq!(v.xzxy(), usizevec4(1_usize, 3_usize, 1_usize, 2_usize));
    assert_eq!(v.xzxz(), usizevec4(1_usize, 3_usize, 1_usize, 3_usize));
    assert_eq!(v.xzyx(), usizevec4(1_usize, 3_usize, 2_usize, 1_usize));
    assert_eq!(v.xzyy(), usizevec4(1_usize, 3_usize, 2_usize, 2_usize));
    assert_eq!(v.xzyz(), usizevec4(1_usize, 3_usize, 2_usize, 3_usize));
    assert_eq!(v.xzzx(), usizevec4(1_usize, 3_usize, 3_usize, 1_usize));
    assert_eq!(v.xzzy(), usizevec4(1_usize, 3_usize, 3_usize, 2_usize));
    assert_eq!(v.xzzz(), usizevec4(1_usize, 3_usize, 3_usize, 3_usize));
    assert_eq!(v.yxxx(), usizevec4(2_usize, 1_usize, 1_usize, 1_usize));
    assert_eq!(v.yxxy(), usizevec4(2_usize, 1_usize, 1_usize, 2_usize));
    assert_eq!(v.yxxz(), usizevec4(2_usize, 1_usize, 1_usize, 3_usize));
    assert_eq!(v.yxyx(), usizevec4(2_usize, 1_usize, 2_usize, 1_usize));
    assert_eq!(v.yxyy(), usizevec4(2_usize, 1_usize, 2_usize, 2_usize));
    assert_eq!(v.yxyz(), usizevec4(2_usize, 1_usize, 2_usize, 3_usize));
    assert_eq!(v.yxzx(), usizevec4(2_usize, 1_usize, 3_usize, 1_usize));
    assert_eq!(v.yxzy(), usizevec4(2_usize, 1_usize, 3_usize, 2_usize));
    assert_eq!(v.yxzz(), usizevec4(2_usize, 1_usize, 3_usize, 3_usize));
    assert_eq!(v.yyxx(), usizevec4(2_usize, 2_usize, 1_usize, 1_usize));
    assert_eq!(v.yyxy(), usizevec4(2_usize, 2_usize, 1_usize, 2_usize));
    assert_eq!(v.yyxz(), usizevec4(2_usize, 2_usize, 1_usize, 3_usize));
    assert_eq!(v.yyyx(), usizevec4(2_usize, 2_usize, 2_usize, 1_usize));
    assert_eq!(v.yyyy(), usizevec4(2_usize, 2_usize, 2_usize, 2_usize));
    assert_eq!(v.yyyz(), usizevec4(2_usize, 2_usize, 2_usize, 3_usize));
    assert_eq!(v.yyzx(), usizevec4(2_usize, 2_usize, 3_usize, 1_usize));
    assert_eq!(v.yyzy(), usizevec4(2_usize, 2_usize, 3_usize, 2_usize));
    assert_eq!(v.yyzz(), usizevec4(2_usize, 2_usize, 3_usize, 3_usize));
    assert_eq!(v.yzxx(), usizevec4(2_usize, 3_usize, 1_usize, 1_usize));
    assert_eq!(v.yzxy(), usizevec4(2_usize, 3_usize, 1_usize, 2_usize));
    assert_eq!(v.yzxz(), usizevec4(2_usize, 3_usize, 1_usize, 3_usize));
    assert_eq!(v.yzyx(), usizevec4(2_usize, 3_usize, 2_usize, 1_usize));
    assert_eq!(v.yzyy(), usizevec4(2_usize, 3_usize, 2_usize, 2_usize));
    assert_eq!(v.yzyz(), usizevec4(2_usize, 3_usize, 2_usize, 3_usize));
    assert_eq!(v.yzzx(), usizevec4(2_usize, 3_usize, 3_usize, 1_usize));
    assert_eq!(v.yzzy(), usizevec4(2_usize, 3_usize, 3_usize, 2_usize));
    assert_eq!(v.yzzz(), usizevec4(2_usize, 3_usize, 3_usize, 3_usize));
    assert_eq!(v.zxxx(), usizevec4(3_usize, 1_usize, 1_usize, 1_usize));
    assert_eq!(v.zxxy(), usizevec4(3_usize, 1_usize, 1_usize, 2_usize));
    assert_eq!(v.zxxz(), usizevec4(3_usize, 1_usize, 1_usize, 3_usize));
    assert_eq!(v.zxyx(), usizevec4(3_usize, 1_usize, 2_usize, 1_usize));
    assert_eq!(v.zxyy(), usizevec4(3_usize, 1_usize, 2_usize, 2_usize));
    assert_eq!(v.zxyz(), usizevec4(3_usize, 1_usize, 2_usize, 3_usize));
    assert_eq!(v.zxzx(), usizevec4(3_usize, 1_usize, 3_usize, 1_usize));
    assert_eq!(v.zxzy(), usizevec4(3_usize, 1_usize, 3_usize, 2_usize));
    assert_eq!(v.zxzz(), usizevec4(3_usize, 1_usize, 3_usize, 3_usize));
    assert_eq!(v.zyxx(), usizevec4(3_usize, 2_usize, 1_usize, 1_usize));
    assert_eq!(v.zyxy(), usizevec4(3_usize, 2_usize, 1_usize, 2_usize));
    assert_eq!(v.zyxz(), usizevec4(3_usize, 2_usize, 1_usize, 3_usize));
    assert_eq!(v.zyyx(), usizevec4(3_usize, 2_usize, 2_usize, 1_usize));
    assert_eq!(v.zyyy(), usizevec4(3_usize, 2_usize, 2_usize, 2_usize));
    assert_eq!(v.zyyz(), usizevec4(3_usize, 2_usize, 2_usize, 3_usize));
    assert_eq!(v.zyzx(), usizevec4(3_usize, 2_usize, 3_usize, 1_usize));
    assert_eq!(v.zyzy(), usizevec4(3_usize, 2_usize, 3_usize, 2_usize));
    assert_eq!(v.zyzz(), usizevec4(3_usize, 2_usize, 3_usize, 3_usize));
    assert_eq!(v.zzxx(), usizevec4(3_usize, 3_usize, 1_usize, 1_usize));
    assert_eq!(v.zzxy(), usizevec4(3_usize, 3_usize, 1_usize, 2_usize));
    assert_eq!(v.zzxz(), usizevec4(3_usize, 3_usize, 1_usize, 3_usize));
    assert_eq!(v.zzyx(), usizevec4(3_usize, 3_usize, 2_usize, 1_usize));
    assert_eq!(v.zzyy(), usizevec4(3_usize, 3_usize, 2_usize, 2_usize));
    assert_eq!(v.zzyz(), usizevec4(3_usize, 3_usize, 2_usize, 3_usize));
    assert_eq!(v.zzzx(), usizevec4(3_usize, 3_usize, 3_usize, 1_usize));
    assert_eq!(v.zzzy(), usizevec4(3_usize, 3_usize, 3_usize, 2_usize));
    assert_eq!(v.zzzz(), usizevec4(3_usize, 3_usize, 3_usize, 3_usize));
    assert_eq!(v.xxx(), usizevec3(1_usize, 1_usize, 1_usize));
    assert_eq!(v.xxy(), usizevec3(1_usize, 1_usize, 2_usize));
    assert_eq!(v.xxz(), usizevec3(1_usize, 1_usize, 3_usize));
    assert_eq!(v.xyx(), usizevec3(1_usize, 2_usize, 1_usize));
    assert_eq!(v.xyy(), usizevec3(1_usize, 2_usize, 2_usize));
    assert_eq!(v.xzx(), usizevec3(1_usize, 3_usize, 1_usize));
    assert_eq!(v.xzy(), usizevec3(1_usize, 3_usize, 2_usize));
    assert_eq!(v.xzz(), usizevec3(1_usize, 3_usize, 3_usize));
    assert_eq!(v.yxx(), usizevec3(2_usize, 1_usize, 1_usize));
    assert_eq!(v.yxy(), usizevec3(2_usize, 1_usize, 2_usize));
    assert_eq!(v.yxz(), usizevec3(2_usize, 1_usize, 3_usize));
    assert_eq!(v.yyx(), usizevec3(2_usize, 2_usize, 1_usize));
    assert_eq!(v.yyy(), usizevec3(2_usize, 2_usize, 2_usize));
    assert_eq!(v.yyz(), usizevec3(2_usize, 2_usize, 3_usize));
    assert_eq!(v.yzx(), usizevec3(2_usize, 3_usize, 1_usize));
    assert_eq!(v.yzy(), usizevec3(2_usize, 3_usize, 2_usize));
    assert_eq!(v.yzz(), usizevec3(2_usize, 3_usize, 3_usize));
    assert_eq!(v.zxx(), usizevec3(3_usize, 1_usize, 1_usize));
    assert_eq!(v.zxy(), usizevec3(3_usize, 1_usize, 2_usize));
    assert_eq!(v.zxz(), usizevec3(3_usize, 1_usize, 3_usize));
    assert_eq!(v.zyx(), usizevec3(3_usize, 2_usize, 1_usize));
    assert_eq!(v.zyy(), usizevec3(3_usize, 2_usize, 2_usize));
    assert_eq!(v.zyz(), usizevec3(3_usize, 2_usize, 3_usize));
    assert_eq!(v.zzx(), usizevec3(3_usize, 3_usize, 1_usize));
    assert_eq!(v.zzy(), usizevec3(3_usize, 3_usize, 2_usize));
    assert_eq!(v.zzz(), usizevec3(3_usize, 3_usize, 3_usize));
    assert_eq!(v.xx(), usizevec2(1_usize, 1_usize));
    assert_eq!(v.xy(), usizevec2(1_usize, 2_usize));
    assert_eq!(v.xz(), usizevec2(1_usize, 3_usize));
    assert_eq!(v.yx(), usizevec2(2_usize, 1_usize));
    assert_eq!(v.yy(), usizevec2(2_usize, 2_usize));
    assert_eq!(v.yz(), usizevec2(2_usize, 3_usize));
    assert_eq!(v.zx(), usizevec2(3_usize, 1_usize));
    assert_eq!(v.zy(), usizevec2(3_usize, 2_usize));
    assert_eq!(v.zz(), usizevec2(3_usize, 3_usize));
    assert_eq!(v.with_xy(rhs2), usizevec3(11_usize, 12_usize, 3_usize));
    assert_eq!(v.with_xz(rhs2), usizevec3(11_usize, 2_usize, 12_usize));
    assert_eq!(v.with_yx(rhs2), usizevec3(12_usize, 11_usize, 3_usize));
    assert_eq!(v.with_yz(rhs2), usizevec3(1_usize, 11_usize, 12_usize));
    assert_eq!(v.with_zx(rhs2), usizevec3(12_usize, 2_usize, 11_usize));
    assert_eq!(v.with_zy(rhs2), usizevec3(1_usize, 12_usize, 11_usize));
});

glam_test!(test_usizevec2_swizzles, {
    let v = usizevec2(1_usize, 2_usize);
    assert_eq!(v, v.xy());
    assert_eq!(v.xxxx(), usizevec4(1_usize, 1_usize, 1_usize, 1_usize));
    assert_eq!(v.xxxy(), usizevec4(1_usize, 1_usize, 1_usize, 2_usize));
    assert_eq!(v.xxyx(), usizevec4(1_usize, 1_usize, 2_usize, 1_usize));
    assert_eq!(v.xxyy(), usizevec4(1_usize, 1_usize, 2_usize, 2_usize));
    assert_eq!(v.xyxx(), usizevec4(1_usize, 2_usize, 1_usize, 1_usize));
    assert_eq!(v.xyxy(), usizevec4(1_usize, 2_usize, 1_usize, 2_usize));
    assert_eq!(v.xyyx(), usizevec4(1_usize, 2_usize, 2_usize, 1_usize));
    assert_eq!(v.xyyy(), usizevec4(1_usize, 2_usize, 2_usize, 2_usize));
    assert_eq!(v.yxxx(), usizevec4(2_usize, 1_usize, 1_usize, 1_usize));
    assert_eq!(v.yxxy(), usizevec4(2_usize, 1_usize, 1_usize, 2_usize));
    assert_eq!(v.yxyx(), usizevec4(2_usize, 1_usize, 2_usize, 1_usize));
    assert_eq!(v.yxyy(), usizevec4(2_usize, 1_usize, 2_usize, 2_usize));
    assert_eq!(v.yyxx(), usizevec4(2_usize, 2_usize, 1_usize, 1_usize));
    assert_eq!(v.yyxy(), usizevec4(2_usize, 2_usize, 1_usize, 2_usize));
    assert_eq!(v.yyyx(), usizevec4(2_usize, 2_usize, 2_usize, 1_usize));
    assert_eq!(v.yyyy(), usizevec4(2_usize, 2_usize, 2_usize, 2_usize));
    assert_eq!(v.xxx(), usizevec3(1_usize, 1_usize, 1_usize));
    assert_eq!(v.xxy(), usizevec3(1_usize, 1_usize, 2_usize));
    assert_eq!(v.xyx(), usizevec3(1_usize, 2_usize, 1_usize));
    assert_eq!(v.xyy(), usizevec3(1_usize, 2_usize, 2_usize));
    assert_eq!(v.yxx(), usizevec3(2_usize, 1_usize, 1_usize));
    assert_eq!(v.yxy(), usizevec3(2_usize, 1_usize, 2_usize));
    assert_eq!(v.yyx(), usizevec3(2_usize, 2_usize, 1_usize));
    assert_eq!(v.yyy(), usizevec3(2_usize, 2_usize, 2_usize));
    assert_eq!(v.xx(), usizevec2(1_usize, 1_usize));
    assert_eq!(v.yx(), usizevec2(2_usize, 1_usize));
    assert_eq!(v.yy(), usizevec2(2_usize, 2_usize));
});
