// Generated by swizzlegen. Do not edit.

use super::Vec2Swizzles;
use crate::{UVec2, UVec3, UVec4, XY, XYZ};

impl Vec2Swizzles for UVec2 {
    type Vec3 = UVec3;
    type Vec4 = UVec4;

    #[inline]
    fn xxxx(self) -> UVec4 {
        UVec4::new(self.x, self.x, self.x, self.x)
    }
    #[inline]
    fn xxxy(self) -> UVec4 {
        UVec4::new(self.x, self.x, self.x, self.y)
    }
    #[inline]
    fn xxyx(self) -> UVec4 {
        UVec4::new(self.x, self.x, self.y, self.x)
    }
    #[inline]
    fn xxyy(self) -> UVec4 {
        UVec4::new(self.x, self.x, self.y, self.y)
    }
    #[inline]
    fn xyxx(self) -> UVec4 {
        UVec4::new(self.x, self.y, self.x, self.x)
    }
    #[inline]
    fn xyxy(self) -> UVec4 {
        UVec4::new(self.x, self.y, self.x, self.y)
    }
    #[inline]
    fn xyyx(self) -> UVec4 {
        UVec4::new(self.x, self.y, self.y, self.x)
    }
    #[inline]
    fn xyyy(self) -> UVec4 {
        UVec4::new(self.x, self.y, self.y, self.y)
    }
    #[inline]
    fn yxxx(self) -> UVec4 {
        UVec4::new(self.y, self.x, self.x, self.x)
    }
    #[inline]
    fn yxxy(self) -> UVec4 {
        UVec4::new(self.y, self.x, self.x, self.y)
    }
    #[inline]
    fn yxyx(self) -> UVec4 {
        UVec4::new(self.y, self.x, self.y, self.x)
    }
    #[inline]
    fn yxyy(self) -> UVec4 {
        UVec4::new(self.y, self.x, self.y, self.y)
    }
    #[inline]
    fn yyxx(self) -> UVec4 {
        UVec4::new(self.y, self.y, self.x, self.x)
    }
    #[inline]
    fn yyxy(self) -> UVec4 {
        UVec4::new(self.y, self.y, self.x, self.y)
    }
    #[inline]
    fn yyyx(self) -> UVec4 {
        UVec4::new(self.y, self.y, self.y, self.x)
    }
    #[inline]
    fn yyyy(self) -> UVec4 {
        UVec4::new(self.y, self.y, self.y, self.y)
    }
    #[inline]
    fn xxx(self) -> UVec3 {
        UVec3(XYZ {
            x: self.x,
            y: self.x,
            z: self.x,
        })
    }
    #[inline]
    fn xxy(self) -> UVec3 {
        UVec3(XYZ {
            x: self.x,
            y: self.x,
            z: self.y,
        })
    }
    #[inline]
    fn xyx(self) -> UVec3 {
        UVec3(XYZ {
            x: self.x,
            y: self.y,
            z: self.x,
        })
    }
    #[inline]
    fn xyy(self) -> UVec3 {
        UVec3(XYZ {
            x: self.x,
            y: self.y,
            z: self.y,
        })
    }
    #[inline]
    fn yxx(self) -> UVec3 {
        UVec3(XYZ {
            x: self.y,
            y: self.x,
            z: self.x,
        })
    }
    #[inline]
    fn yxy(self) -> UVec3 {
        UVec3(XYZ {
            x: self.y,
            y: self.x,
            z: self.y,
        })
    }
    #[inline]
    fn yyx(self) -> UVec3 {
        UVec3(XYZ {
            x: self.y,
            y: self.y,
            z: self.x,
        })
    }
    #[inline]
    fn yyy(self) -> UVec3 {
        UVec3(XYZ {
            x: self.y,
            y: self.y,
            z: self.y,
        })
    }
    #[inline]
    fn xx(self) -> Self {
        Self(XY {
            x: self.x,
            y: self.x,
        })
    }
    #[inline]
    fn yx(self) -> Self {
        Self(XY {
            x: self.y,
            y: self.x,
        })
    }
    #[inline]
    fn yy(self) -> Self {
        Self(XY {
            x: self.y,
            y: self.y,
        })
    }
}
