// Generated from vec.rs.tera template. Edit the template, not the generated file.

use crate::{
    BVec3, BVec3A, I16Vec3, I64Vec3, I8Vec3, IVec3, U16Vec3, U64Vec3, U8Vec3, UVec2, UVec4,
};

use core::fmt;
use core::iter::{Product, Sum};
use core::{f32, ops::*};

/// Creates a 3-dimensional vector.
#[inline(always)]
#[must_use]
pub const fn uvec3(x: u32, y: u32, z: u32) -> UVec3 {
    UVec3::new(x, y, z)
}

/// A 3-dimensional vector.
#[cfg_attr(not(target_arch = "spirv"), derive(Hash))]
#[derive(Clone, Copy, PartialEq, Eq)]
#[cfg_attr(not(target_arch = "spirv"), repr(C))]
#[cfg_attr(target_arch = "spirv", repr(simd))]
pub struct UVec3 {
    pub x: u32,
    pub y: u32,
    pub z: u32,
}

impl UVec3 {
    /// All zeroes.
    pub const ZERO: Self = Self::splat(0);

    /// All ones.
    pub const ONE: Self = Self::splat(1);

    /// All `u32::MIN`.
    pub const MIN: Self = Self::splat(u32::MIN);

    /// All `u32::MAX`.
    pub const MAX: Self = Self::splat(u32::MAX);

    /// A unit vector pointing along the positive X axis.
    pub const X: Self = Self::new(1, 0, 0);

    /// A unit vector pointing along the positive Y axis.
    pub const Y: Self = Self::new(0, 1, 0);

    /// A unit vector pointing along the positive Z axis.
    pub const Z: Self = Self::new(0, 0, 1);

    /// The unit axes.
    pub const AXES: [Self; 3] = [Self::X, Self::Y, Self::Z];

    /// Creates a new vector.
    #[inline(always)]
    #[must_use]
    pub const fn new(x: u32, y: u32, z: u32) -> Self {
        Self { x, y, z }
    }

    /// Creates a vector with all elements set to `v`.
    #[inline]
    #[must_use]
    pub const fn splat(v: u32) -> Self {
        Self { x: v, y: v, z: v }
    }

    /// Returns a vector containing each element of `self` modified by a mapping function `f`.
    #[inline]
    #[must_use]
    pub fn map<F>(self, f: F) -> Self
    where
        F: Fn(u32) -> u32,
    {
        Self::new(f(self.x), f(self.y), f(self.z))
    }

    /// Creates a vector from the elements in `if_true` and `if_false`, selecting which to use
    /// for each element of `self`.
    ///
    /// A true element in the mask uses the corresponding element from `if_true`, and false
    /// uses the element from `if_false`.
    #[inline]
    #[must_use]
    pub fn select(mask: BVec3, if_true: Self, if_false: Self) -> Self {
        Self {
            x: if mask.test(0) { if_true.x } else { if_false.x },
            y: if mask.test(1) { if_true.y } else { if_false.y },
            z: if mask.test(2) { if_true.z } else { if_false.z },
        }
    }

    /// Creates a new vector from an array.
    #[inline]
    #[must_use]
    pub const fn from_array(a: [u32; 3]) -> Self {
        Self::new(a[0], a[1], a[2])
    }

    /// `[x, y, z]`
    #[inline]
    #[must_use]
    pub const fn to_array(&self) -> [u32; 3] {
        [self.x, self.y, self.z]
    }

    /// Creates a vector from the first 3 values in `slice`.
    ///
    /// # Panics
    ///
    /// Panics if `slice` is less than 3 elements long.
    #[inline]
    #[must_use]
    pub const fn from_slice(slice: &[u32]) -> Self {
        assert!(slice.len() >= 3);
        Self::new(slice[0], slice[1], slice[2])
    }

    /// Writes the elements of `self` to the first 3 elements in `slice`.
    ///
    /// # Panics
    ///
    /// Panics if `slice` is less than 3 elements long.
    #[inline]
    pub fn write_to_slice(self, slice: &mut [u32]) {
        slice[..3].copy_from_slice(&self.to_array());
    }

    /// Internal method for creating a 3D vector from a 4D vector, discarding `w`.
    #[allow(dead_code)]
    #[inline]
    #[must_use]
    pub(crate) fn from_vec4(v: UVec4) -> Self {
        Self {
            x: v.x,
            y: v.y,
            z: v.z,
        }
    }

    /// Creates a 4D vector from `self` and the given `w` value.
    #[inline]
    #[must_use]
    pub fn extend(self, w: u32) -> UVec4 {
        UVec4::new(self.x, self.y, self.z, w)
    }

    /// Creates a 2D vector from the `x` and `y` elements of `self`, discarding `z`.
    ///
    /// Truncation may also be performed by using [`self.xy()`][crate::swizzles::Vec3Swizzles::xy()].
    #[inline]
    #[must_use]
    pub fn truncate(self) -> UVec2 {
        use crate::swizzles::Vec3Swizzles;
        self.xy()
    }

    /// Creates a 3D vector from `self` with the given value of `x`.
    #[inline]
    #[must_use]
    pub fn with_x(mut self, x: u32) -> Self {
        self.x = x;
        self
    }

    /// Creates a 3D vector from `self` with the given value of `y`.
    #[inline]
    #[must_use]
    pub fn with_y(mut self, y: u32) -> Self {
        self.y = y;
        self
    }

    /// Creates a 3D vector from `self` with the given value of `z`.
    #[inline]
    #[must_use]
    pub fn with_z(mut self, z: u32) -> Self {
        self.z = z;
        self
    }

    /// Computes the dot product of `self` and `rhs`.
    #[inline]
    #[must_use]
    pub fn dot(self, rhs: Self) -> u32 {
        (self.x * rhs.x) + (self.y * rhs.y) + (self.z * rhs.z)
    }

    /// Returns a vector where every component is the dot product of `self` and `rhs`.
    #[inline]
    #[must_use]
    pub fn dot_into_vec(self, rhs: Self) -> Self {
        Self::splat(self.dot(rhs))
    }

    /// Computes the cross product of `self` and `rhs`.
    #[inline]
    #[must_use]
    pub fn cross(self, rhs: Self) -> Self {
        Self {
            x: self.y * rhs.z - rhs.y * self.z,
            y: self.z * rhs.x - rhs.z * self.x,
            z: self.x * rhs.y - rhs.x * self.y,
        }
    }

    /// Returns a vector containing the minimum values for each element of `self` and `rhs`.
    ///
    /// In other words this computes `[self.x.min(rhs.x), self.y.min(rhs.y), ..]`.
    #[inline]
    #[must_use]
    pub fn min(self, rhs: Self) -> Self {
        Self {
            x: self.x.min(rhs.x),
            y: self.y.min(rhs.y),
            z: self.z.min(rhs.z),
        }
    }

    /// Returns a vector containing the maximum values for each element of `self` and `rhs`.
    ///
    /// In other words this computes `[self.x.max(rhs.x), self.y.max(rhs.y), ..]`.
    #[inline]
    #[must_use]
    pub fn max(self, rhs: Self) -> Self {
        Self {
            x: self.x.max(rhs.x),
            y: self.y.max(rhs.y),
            z: self.z.max(rhs.z),
        }
    }

    /// Component-wise clamping of values, similar to [`u32::clamp`].
    ///
    /// Each element in `min` must be less-or-equal to the corresponding element in `max`.
    ///
    /// # Panics
    ///
    /// Will panic if `min` is greater than `max` when `glam_assert` is enabled.
    #[inline]
    #[must_use]
    pub fn clamp(self, min: Self, max: Self) -> Self {
        glam_assert!(min.cmple(max).all(), "clamp: expected min <= max");
        self.max(min).min(max)
    }

    /// Returns the horizontal minimum of `self`.
    ///
    /// In other words this computes `min(x, y, ..)`.
    #[inline]
    #[must_use]
    pub fn min_element(self) -> u32 {
        self.x.min(self.y.min(self.z))
    }

    /// Returns the horizontal maximum of `self`.
    ///
    /// In other words this computes `max(x, y, ..)`.
    #[inline]
    #[must_use]
    pub fn max_element(self) -> u32 {
        self.x.max(self.y.max(self.z))
    }

    /// Returns the sum of all elements of `self`.
    ///
    /// In other words, this computes `self.x + self.y + ..`.
    #[inline]
    #[must_use]
    pub fn element_sum(self) -> u32 {
        self.x + self.y + self.z
    }

    /// Returns the product of all elements of `self`.
    ///
    /// In other words, this computes `self.x * self.y * ..`.
    #[inline]
    #[must_use]
    pub fn element_product(self) -> u32 {
        self.x * self.y * self.z
    }

    /// Returns a vector mask containing the result of a `==` comparison for each element of
    /// `self` and `rhs`.
    ///
    /// In other words, this computes `[self.x == rhs.x, self.y == rhs.y, ..]` for all
    /// elements.
    #[inline]
    #[must_use]
    pub fn cmpeq(self, rhs: Self) -> BVec3 {
        BVec3::new(self.x.eq(&rhs.x), self.y.eq(&rhs.y), self.z.eq(&rhs.z))
    }

    /// Returns a vector mask containing the result of a `!=` comparison for each element of
    /// `self` and `rhs`.
    ///
    /// In other words this computes `[self.x != rhs.x, self.y != rhs.y, ..]` for all
    /// elements.
    #[inline]
    #[must_use]
    pub fn cmpne(self, rhs: Self) -> BVec3 {
        BVec3::new(self.x.ne(&rhs.x), self.y.ne(&rhs.y), self.z.ne(&rhs.z))
    }

    /// Returns a vector mask containing the result of a `>=` comparison for each element of
    /// `self` and `rhs`.
    ///
    /// In other words this computes `[self.x >= rhs.x, self.y >= rhs.y, ..]` for all
    /// elements.
    #[inline]
    #[must_use]
    pub fn cmpge(self, rhs: Self) -> BVec3 {
        BVec3::new(self.x.ge(&rhs.x), self.y.ge(&rhs.y), self.z.ge(&rhs.z))
    }

    /// Returns a vector mask containing the result of a `>` comparison for each element of
    /// `self` and `rhs`.
    ///
    /// In other words this computes `[self.x > rhs.x, self.y > rhs.y, ..]` for all
    /// elements.
    #[inline]
    #[must_use]
    pub fn cmpgt(self, rhs: Self) -> BVec3 {
        BVec3::new(self.x.gt(&rhs.x), self.y.gt(&rhs.y), self.z.gt(&rhs.z))
    }

    /// Returns a vector mask containing the result of a `<=` comparison for each element of
    /// `self` and `rhs`.
    ///
    /// In other words this computes `[self.x <= rhs.x, self.y <= rhs.y, ..]` for all
    /// elements.
    #[inline]
    #[must_use]
    pub fn cmple(self, rhs: Self) -> BVec3 {
        BVec3::new(self.x.le(&rhs.x), self.y.le(&rhs.y), self.z.le(&rhs.z))
    }

    /// Returns a vector mask containing the result of a `<` comparison for each element of
    /// `self` and `rhs`.
    ///
    /// In other words this computes `[self.x < rhs.x, self.y < rhs.y, ..]` for all
    /// elements.
    #[inline]
    #[must_use]
    pub fn cmplt(self, rhs: Self) -> BVec3 {
        BVec3::new(self.x.lt(&rhs.x), self.y.lt(&rhs.y), self.z.lt(&rhs.z))
    }

    /// Computes the squared length of `self`.
    #[doc(alias = "magnitude2")]
    #[inline]
    #[must_use]
    pub fn length_squared(self) -> u32 {
        self.dot(self)
    }

    /// Casts all elements of `self` to `f32`.
    #[inline]
    #[must_use]
    pub fn as_vec3(&self) -> crate::Vec3 {
        crate::Vec3::new(self.x as f32, self.y as f32, self.z as f32)
    }

    /// Casts all elements of `self` to `f32`.
    #[inline]
    #[must_use]
    pub fn as_vec3a(&self) -> crate::Vec3A {
        crate::Vec3A::new(self.x as f32, self.y as f32, self.z as f32)
    }

    /// Casts all elements of `self` to `f64`.
    #[inline]
    #[must_use]
    pub fn as_dvec3(&self) -> crate::DVec3 {
        crate::DVec3::new(self.x as f64, self.y as f64, self.z as f64)
    }

    /// Casts all elements of `self` to `i8`.
    #[inline]
    #[must_use]
    pub fn as_i8vec3(&self) -> crate::I8Vec3 {
        crate::I8Vec3::new(self.x as i8, self.y as i8, self.z as i8)
    }

    /// Casts all elements of `self` to `u8`.
    #[inline]
    #[must_use]
    pub fn as_u8vec3(&self) -> crate::U8Vec3 {
        crate::U8Vec3::new(self.x as u8, self.y as u8, self.z as u8)
    }

    /// Casts all elements of `self` to `i16`.
    #[inline]
    #[must_use]
    pub fn as_i16vec3(&self) -> crate::I16Vec3 {
        crate::I16Vec3::new(self.x as i16, self.y as i16, self.z as i16)
    }

    /// Casts all elements of `self` to `u16`.
    #[inline]
    #[must_use]
    pub fn as_u16vec3(&self) -> crate::U16Vec3 {
        crate::U16Vec3::new(self.x as u16, self.y as u16, self.z as u16)
    }

    /// Casts all elements of `self` to `i32`.
    #[inline]
    #[must_use]
    pub fn as_ivec3(&self) -> crate::IVec3 {
        crate::IVec3::new(self.x as i32, self.y as i32, self.z as i32)
    }

    /// Casts all elements of `self` to `i64`.
    #[inline]
    #[must_use]
    pub fn as_i64vec3(&self) -> crate::I64Vec3 {
        crate::I64Vec3::new(self.x as i64, self.y as i64, self.z as i64)
    }

    /// Casts all elements of `self` to `u64`.
    #[inline]
    #[must_use]
    pub fn as_u64vec3(&self) -> crate::U64Vec3 {
        crate::U64Vec3::new(self.x as u64, self.y as u64, self.z as u64)
    }

    /// Returns a vector containing the wrapping addition of `self` and `rhs`.
    ///
    /// In other words this computes `[self.x.wrapping_add(rhs.x), self.y.wrapping_add(rhs.y), ..]`.
    #[inline]
    #[must_use]
    pub const fn wrapping_add(self, rhs: Self) -> Self {
        Self {
            x: self.x.wrapping_add(rhs.x),
            y: self.y.wrapping_add(rhs.y),
            z: self.z.wrapping_add(rhs.z),
        }
    }

    /// Returns a vector containing the wrapping subtraction of `self` and `rhs`.
    ///
    /// In other words this computes `[self.x.wrapping_sub(rhs.x), self.y.wrapping_sub(rhs.y), ..]`.
    #[inline]
    #[must_use]
    pub const fn wrapping_sub(self, rhs: Self) -> Self {
        Self {
            x: self.x.wrapping_sub(rhs.x),
            y: self.y.wrapping_sub(rhs.y),
            z: self.z.wrapping_sub(rhs.z),
        }
    }

    /// Returns a vector containing the wrapping multiplication of `self` and `rhs`.
    ///
    /// In other words this computes `[self.x.wrapping_mul(rhs.x), self.y.wrapping_mul(rhs.y), ..]`.
    #[inline]
    #[must_use]
    pub const fn wrapping_mul(self, rhs: Self) -> Self {
        Self {
            x: self.x.wrapping_mul(rhs.x),
            y: self.y.wrapping_mul(rhs.y),
            z: self.z.wrapping_mul(rhs.z),
        }
    }

    /// Returns a vector containing the wrapping division of `self` and `rhs`.
    ///
    /// In other words this computes `[self.x.wrapping_div(rhs.x), self.y.wrapping_div(rhs.y), ..]`.
    #[inline]
    #[must_use]
    pub const fn wrapping_div(self, rhs: Self) -> Self {
        Self {
            x: self.x.wrapping_div(rhs.x),
            y: self.y.wrapping_div(rhs.y),
            z: self.z.wrapping_div(rhs.z),
        }
    }

    /// Returns a vector containing the saturating addition of `self` and `rhs`.
    ///
    /// In other words this computes `[self.x.saturating_add(rhs.x), self.y.saturating_add(rhs.y), ..]`.
    #[inline]
    #[must_use]
    pub const fn saturating_add(self, rhs: Self) -> Self {
        Self {
            x: self.x.saturating_add(rhs.x),
            y: self.y.saturating_add(rhs.y),
            z: self.z.saturating_add(rhs.z),
        }
    }

    /// Returns a vector containing the saturating subtraction of `self` and `rhs`.
    ///
    /// In other words this computes `[self.x.saturating_sub(rhs.x), self.y.saturating_sub(rhs.y), ..]`.
    #[inline]
    #[must_use]
    pub const fn saturating_sub(self, rhs: Self) -> Self {
        Self {
            x: self.x.saturating_sub(rhs.x),
            y: self.y.saturating_sub(rhs.y),
            z: self.z.saturating_sub(rhs.z),
        }
    }

    /// Returns a vector containing the saturating multiplication of `self` and `rhs`.
    ///
    /// In other words this computes `[self.x.saturating_mul(rhs.x), self.y.saturating_mul(rhs.y), ..]`.
    #[inline]
    #[must_use]
    pub const fn saturating_mul(self, rhs: Self) -> Self {
        Self {
            x: self.x.saturating_mul(rhs.x),
            y: self.y.saturating_mul(rhs.y),
            z: self.z.saturating_mul(rhs.z),
        }
    }

    /// Returns a vector containing the saturating division of `self` and `rhs`.
    ///
    /// In other words this computes `[self.x.saturating_div(rhs.x), self.y.saturating_div(rhs.y), ..]`.
    #[inline]
    #[must_use]
    pub const fn saturating_div(self, rhs: Self) -> Self {
        Self {
            x: self.x.saturating_div(rhs.x),
            y: self.y.saturating_div(rhs.y),
            z: self.z.saturating_div(rhs.z),
        }
    }

    /// Returns a vector containing the wrapping addition of `self` and signed vector `rhs`.
    ///
    /// In other words this computes `[self.x.wrapping_add_signed(rhs.x), self.y.wrapping_add_signed(rhs.y), ..]`.
    #[inline]
    #[must_use]
    pub const fn wrapping_add_signed(self, rhs: IVec3) -> Self {
        Self {
            x: self.x.wrapping_add_signed(rhs.x),
            y: self.y.wrapping_add_signed(rhs.y),
            z: self.z.wrapping_add_signed(rhs.z),
        }
    }

    /// Returns a vector containing the saturating addition of `self` and signed vector `rhs`.
    ///
    /// In other words this computes `[self.x.saturating_add_signed(rhs.x), self.y.saturating_add_signed(rhs.y), ..]`.
    #[inline]
    #[must_use]
    pub const fn saturating_add_signed(self, rhs: IVec3) -> Self {
        Self {
            x: self.x.saturating_add_signed(rhs.x),
            y: self.y.saturating_add_signed(rhs.y),
            z: self.z.saturating_add_signed(rhs.z),
        }
    }
}

impl Default for UVec3 {
    #[inline(always)]
    fn default() -> Self {
        Self::ZERO
    }
}

impl Div<UVec3> for UVec3 {
    type Output = Self;
    #[inline]
    fn div(self, rhs: Self) -> Self {
        Self {
            x: self.x.div(rhs.x),
            y: self.y.div(rhs.y),
            z: self.z.div(rhs.z),
        }
    }
}

impl Div<&UVec3> for UVec3 {
    type Output = UVec3;
    #[inline]
    fn div(self, rhs: &UVec3) -> UVec3 {
        self.div(*rhs)
    }
}

impl Div<&UVec3> for &UVec3 {
    type Output = UVec3;
    #[inline]
    fn div(self, rhs: &UVec3) -> UVec3 {
        (*self).div(*rhs)
    }
}

impl Div<UVec3> for &UVec3 {
    type Output = UVec3;
    #[inline]
    fn div(self, rhs: UVec3) -> UVec3 {
        (*self).div(rhs)
    }
}

impl DivAssign<UVec3> for UVec3 {
    #[inline]
    fn div_assign(&mut self, rhs: Self) {
        self.x.div_assign(rhs.x);
        self.y.div_assign(rhs.y);
        self.z.div_assign(rhs.z);
    }
}

impl DivAssign<&Self> for UVec3 {
    #[inline]
    fn div_assign(&mut self, rhs: &Self) {
        self.div_assign(*rhs)
    }
}

impl Div<u32> for UVec3 {
    type Output = Self;
    #[inline]
    fn div(self, rhs: u32) -> Self {
        Self {
            x: self.x.div(rhs),
            y: self.y.div(rhs),
            z: self.z.div(rhs),
        }
    }
}

impl Div<&u32> for UVec3 {
    type Output = UVec3;
    #[inline]
    fn div(self, rhs: &u32) -> UVec3 {
        self.div(*rhs)
    }
}

impl Div<&u32> for &UVec3 {
    type Output = UVec3;
    #[inline]
    fn div(self, rhs: &u32) -> UVec3 {
        (*self).div(*rhs)
    }
}

impl Div<u32> for &UVec3 {
    type Output = UVec3;
    #[inline]
    fn div(self, rhs: u32) -> UVec3 {
        (*self).div(rhs)
    }
}

impl DivAssign<u32> for UVec3 {
    #[inline]
    fn div_assign(&mut self, rhs: u32) {
        self.x.div_assign(rhs);
        self.y.div_assign(rhs);
        self.z.div_assign(rhs);
    }
}

impl DivAssign<&u32> for UVec3 {
    #[inline]
    fn div_assign(&mut self, rhs: &u32) {
        self.div_assign(*rhs)
    }
}

impl Div<UVec3> for u32 {
    type Output = UVec3;
    #[inline]
    fn div(self, rhs: UVec3) -> UVec3 {
        UVec3 {
            x: self.div(rhs.x),
            y: self.div(rhs.y),
            z: self.div(rhs.z),
        }
    }
}

impl Div<&UVec3> for u32 {
    type Output = UVec3;
    #[inline]
    fn div(self, rhs: &UVec3) -> UVec3 {
        self.div(*rhs)
    }
}

impl Div<&UVec3> for &u32 {
    type Output = UVec3;
    #[inline]
    fn div(self, rhs: &UVec3) -> UVec3 {
        (*self).div(*rhs)
    }
}

impl Div<UVec3> for &u32 {
    type Output = UVec3;
    #[inline]
    fn div(self, rhs: UVec3) -> UVec3 {
        (*self).div(rhs)
    }
}

impl Mul<UVec3> for UVec3 {
    type Output = Self;
    #[inline]
    fn mul(self, rhs: Self) -> Self {
        Self {
            x: self.x.mul(rhs.x),
            y: self.y.mul(rhs.y),
            z: self.z.mul(rhs.z),
        }
    }
}

impl Mul<&UVec3> for UVec3 {
    type Output = UVec3;
    #[inline]
    fn mul(self, rhs: &UVec3) -> UVec3 {
        self.mul(*rhs)
    }
}

impl Mul<&UVec3> for &UVec3 {
    type Output = UVec3;
    #[inline]
    fn mul(self, rhs: &UVec3) -> UVec3 {
        (*self).mul(*rhs)
    }
}

impl Mul<UVec3> for &UVec3 {
    type Output = UVec3;
    #[inline]
    fn mul(self, rhs: UVec3) -> UVec3 {
        (*self).mul(rhs)
    }
}

impl MulAssign<UVec3> for UVec3 {
    #[inline]
    fn mul_assign(&mut self, rhs: Self) {
        self.x.mul_assign(rhs.x);
        self.y.mul_assign(rhs.y);
        self.z.mul_assign(rhs.z);
    }
}

impl MulAssign<&Self> for UVec3 {
    #[inline]
    fn mul_assign(&mut self, rhs: &Self) {
        self.mul_assign(*rhs)
    }
}

impl Mul<u32> for UVec3 {
    type Output = Self;
    #[inline]
    fn mul(self, rhs: u32) -> Self {
        Self {
            x: self.x.mul(rhs),
            y: self.y.mul(rhs),
            z: self.z.mul(rhs),
        }
    }
}

impl Mul<&u32> for UVec3 {
    type Output = UVec3;
    #[inline]
    fn mul(self, rhs: &u32) -> UVec3 {
        self.mul(*rhs)
    }
}

impl Mul<&u32> for &UVec3 {
    type Output = UVec3;
    #[inline]
    fn mul(self, rhs: &u32) -> UVec3 {
        (*self).mul(*rhs)
    }
}

impl Mul<u32> for &UVec3 {
    type Output = UVec3;
    #[inline]
    fn mul(self, rhs: u32) -> UVec3 {
        (*self).mul(rhs)
    }
}

impl MulAssign<u32> for UVec3 {
    #[inline]
    fn mul_assign(&mut self, rhs: u32) {
        self.x.mul_assign(rhs);
        self.y.mul_assign(rhs);
        self.z.mul_assign(rhs);
    }
}

impl MulAssign<&u32> for UVec3 {
    #[inline]
    fn mul_assign(&mut self, rhs: &u32) {
        self.mul_assign(*rhs)
    }
}

impl Mul<UVec3> for u32 {
    type Output = UVec3;
    #[inline]
    fn mul(self, rhs: UVec3) -> UVec3 {
        UVec3 {
            x: self.mul(rhs.x),
            y: self.mul(rhs.y),
            z: self.mul(rhs.z),
        }
    }
}

impl Mul<&UVec3> for u32 {
    type Output = UVec3;
    #[inline]
    fn mul(self, rhs: &UVec3) -> UVec3 {
        self.mul(*rhs)
    }
}

impl Mul<&UVec3> for &u32 {
    type Output = UVec3;
    #[inline]
    fn mul(self, rhs: &UVec3) -> UVec3 {
        (*self).mul(*rhs)
    }
}

impl Mul<UVec3> for &u32 {
    type Output = UVec3;
    #[inline]
    fn mul(self, rhs: UVec3) -> UVec3 {
        (*self).mul(rhs)
    }
}

impl Add<UVec3> for UVec3 {
    type Output = Self;
    #[inline]
    fn add(self, rhs: Self) -> Self {
        Self {
            x: self.x.add(rhs.x),
            y: self.y.add(rhs.y),
            z: self.z.add(rhs.z),
        }
    }
}

impl Add<&UVec3> for UVec3 {
    type Output = UVec3;
    #[inline]
    fn add(self, rhs: &UVec3) -> UVec3 {
        self.add(*rhs)
    }
}

impl Add<&UVec3> for &UVec3 {
    type Output = UVec3;
    #[inline]
    fn add(self, rhs: &UVec3) -> UVec3 {
        (*self).add(*rhs)
    }
}

impl Add<UVec3> for &UVec3 {
    type Output = UVec3;
    #[inline]
    fn add(self, rhs: UVec3) -> UVec3 {
        (*self).add(rhs)
    }
}

impl AddAssign<UVec3> for UVec3 {
    #[inline]
    fn add_assign(&mut self, rhs: Self) {
        self.x.add_assign(rhs.x);
        self.y.add_assign(rhs.y);
        self.z.add_assign(rhs.z);
    }
}

impl AddAssign<&Self> for UVec3 {
    #[inline]
    fn add_assign(&mut self, rhs: &Self) {
        self.add_assign(*rhs)
    }
}

impl Add<u32> for UVec3 {
    type Output = Self;
    #[inline]
    fn add(self, rhs: u32) -> Self {
        Self {
            x: self.x.add(rhs),
            y: self.y.add(rhs),
            z: self.z.add(rhs),
        }
    }
}

impl Add<&u32> for UVec3 {
    type Output = UVec3;
    #[inline]
    fn add(self, rhs: &u32) -> UVec3 {
        self.add(*rhs)
    }
}

impl Add<&u32> for &UVec3 {
    type Output = UVec3;
    #[inline]
    fn add(self, rhs: &u32) -> UVec3 {
        (*self).add(*rhs)
    }
}

impl Add<u32> for &UVec3 {
    type Output = UVec3;
    #[inline]
    fn add(self, rhs: u32) -> UVec3 {
        (*self).add(rhs)
    }
}

impl AddAssign<u32> for UVec3 {
    #[inline]
    fn add_assign(&mut self, rhs: u32) {
        self.x.add_assign(rhs);
        self.y.add_assign(rhs);
        self.z.add_assign(rhs);
    }
}

impl AddAssign<&u32> for UVec3 {
    #[inline]
    fn add_assign(&mut self, rhs: &u32) {
        self.add_assign(*rhs)
    }
}

impl Add<UVec3> for u32 {
    type Output = UVec3;
    #[inline]
    fn add(self, rhs: UVec3) -> UVec3 {
        UVec3 {
            x: self.add(rhs.x),
            y: self.add(rhs.y),
            z: self.add(rhs.z),
        }
    }
}

impl Add<&UVec3> for u32 {
    type Output = UVec3;
    #[inline]
    fn add(self, rhs: &UVec3) -> UVec3 {
        self.add(*rhs)
    }
}

impl Add<&UVec3> for &u32 {
    type Output = UVec3;
    #[inline]
    fn add(self, rhs: &UVec3) -> UVec3 {
        (*self).add(*rhs)
    }
}

impl Add<UVec3> for &u32 {
    type Output = UVec3;
    #[inline]
    fn add(self, rhs: UVec3) -> UVec3 {
        (*self).add(rhs)
    }
}

impl Sub<UVec3> for UVec3 {
    type Output = Self;
    #[inline]
    fn sub(self, rhs: Self) -> Self {
        Self {
            x: self.x.sub(rhs.x),
            y: self.y.sub(rhs.y),
            z: self.z.sub(rhs.z),
        }
    }
}

impl Sub<&UVec3> for UVec3 {
    type Output = UVec3;
    #[inline]
    fn sub(self, rhs: &UVec3) -> UVec3 {
        self.sub(*rhs)
    }
}

impl Sub<&UVec3> for &UVec3 {
    type Output = UVec3;
    #[inline]
    fn sub(self, rhs: &UVec3) -> UVec3 {
        (*self).sub(*rhs)
    }
}

impl Sub<UVec3> for &UVec3 {
    type Output = UVec3;
    #[inline]
    fn sub(self, rhs: UVec3) -> UVec3 {
        (*self).sub(rhs)
    }
}

impl SubAssign<UVec3> for UVec3 {
    #[inline]
    fn sub_assign(&mut self, rhs: UVec3) {
        self.x.sub_assign(rhs.x);
        self.y.sub_assign(rhs.y);
        self.z.sub_assign(rhs.z);
    }
}

impl SubAssign<&Self> for UVec3 {
    #[inline]
    fn sub_assign(&mut self, rhs: &Self) {
        self.sub_assign(*rhs)
    }
}

impl Sub<u32> for UVec3 {
    type Output = Self;
    #[inline]
    fn sub(self, rhs: u32) -> Self {
        Self {
            x: self.x.sub(rhs),
            y: self.y.sub(rhs),
            z: self.z.sub(rhs),
        }
    }
}

impl Sub<&u32> for UVec3 {
    type Output = UVec3;
    #[inline]
    fn sub(self, rhs: &u32) -> UVec3 {
        self.sub(*rhs)
    }
}

impl Sub<&u32> for &UVec3 {
    type Output = UVec3;
    #[inline]
    fn sub(self, rhs: &u32) -> UVec3 {
        (*self).sub(*rhs)
    }
}

impl Sub<u32> for &UVec3 {
    type Output = UVec3;
    #[inline]
    fn sub(self, rhs: u32) -> UVec3 {
        (*self).sub(rhs)
    }
}

impl SubAssign<u32> for UVec3 {
    #[inline]
    fn sub_assign(&mut self, rhs: u32) {
        self.x.sub_assign(rhs);
        self.y.sub_assign(rhs);
        self.z.sub_assign(rhs);
    }
}

impl SubAssign<&u32> for UVec3 {
    #[inline]
    fn sub_assign(&mut self, rhs: &u32) {
        self.sub_assign(*rhs)
    }
}

impl Sub<UVec3> for u32 {
    type Output = UVec3;
    #[inline]
    fn sub(self, rhs: UVec3) -> UVec3 {
        UVec3 {
            x: self.sub(rhs.x),
            y: self.sub(rhs.y),
            z: self.sub(rhs.z),
        }
    }
}

impl Sub<&UVec3> for u32 {
    type Output = UVec3;
    #[inline]
    fn sub(self, rhs: &UVec3) -> UVec3 {
        self.sub(*rhs)
    }
}

impl Sub<&UVec3> for &u32 {
    type Output = UVec3;
    #[inline]
    fn sub(self, rhs: &UVec3) -> UVec3 {
        (*self).sub(*rhs)
    }
}

impl Sub<UVec3> for &u32 {
    type Output = UVec3;
    #[inline]
    fn sub(self, rhs: UVec3) -> UVec3 {
        (*self).sub(rhs)
    }
}

impl Rem<UVec3> for UVec3 {
    type Output = Self;
    #[inline]
    fn rem(self, rhs: Self) -> Self {
        Self {
            x: self.x.rem(rhs.x),
            y: self.y.rem(rhs.y),
            z: self.z.rem(rhs.z),
        }
    }
}

impl Rem<&UVec3> for UVec3 {
    type Output = UVec3;
    #[inline]
    fn rem(self, rhs: &UVec3) -> UVec3 {
        self.rem(*rhs)
    }
}

impl Rem<&UVec3> for &UVec3 {
    type Output = UVec3;
    #[inline]
    fn rem(self, rhs: &UVec3) -> UVec3 {
        (*self).rem(*rhs)
    }
}

impl Rem<UVec3> for &UVec3 {
    type Output = UVec3;
    #[inline]
    fn rem(self, rhs: UVec3) -> UVec3 {
        (*self).rem(rhs)
    }
}

impl RemAssign<UVec3> for UVec3 {
    #[inline]
    fn rem_assign(&mut self, rhs: Self) {
        self.x.rem_assign(rhs.x);
        self.y.rem_assign(rhs.y);
        self.z.rem_assign(rhs.z);
    }
}

impl RemAssign<&Self> for UVec3 {
    #[inline]
    fn rem_assign(&mut self, rhs: &Self) {
        self.rem_assign(*rhs)
    }
}

impl Rem<u32> for UVec3 {
    type Output = Self;
    #[inline]
    fn rem(self, rhs: u32) -> Self {
        Self {
            x: self.x.rem(rhs),
            y: self.y.rem(rhs),
            z: self.z.rem(rhs),
        }
    }
}

impl Rem<&u32> for UVec3 {
    type Output = UVec3;
    #[inline]
    fn rem(self, rhs: &u32) -> UVec3 {
        self.rem(*rhs)
    }
}

impl Rem<&u32> for &UVec3 {
    type Output = UVec3;
    #[inline]
    fn rem(self, rhs: &u32) -> UVec3 {
        (*self).rem(*rhs)
    }
}

impl Rem<u32> for &UVec3 {
    type Output = UVec3;
    #[inline]
    fn rem(self, rhs: u32) -> UVec3 {
        (*self).rem(rhs)
    }
}

impl RemAssign<u32> for UVec3 {
    #[inline]
    fn rem_assign(&mut self, rhs: u32) {
        self.x.rem_assign(rhs);
        self.y.rem_assign(rhs);
        self.z.rem_assign(rhs);
    }
}

impl RemAssign<&u32> for UVec3 {
    #[inline]
    fn rem_assign(&mut self, rhs: &u32) {
        self.rem_assign(*rhs)
    }
}

impl Rem<UVec3> for u32 {
    type Output = UVec3;
    #[inline]
    fn rem(self, rhs: UVec3) -> UVec3 {
        UVec3 {
            x: self.rem(rhs.x),
            y: self.rem(rhs.y),
            z: self.rem(rhs.z),
        }
    }
}

impl Rem<&UVec3> for u32 {
    type Output = UVec3;
    #[inline]
    fn rem(self, rhs: &UVec3) -> UVec3 {
        self.rem(*rhs)
    }
}

impl Rem<&UVec3> for &u32 {
    type Output = UVec3;
    #[inline]
    fn rem(self, rhs: &UVec3) -> UVec3 {
        (*self).rem(*rhs)
    }
}

impl Rem<UVec3> for &u32 {
    type Output = UVec3;
    #[inline]
    fn rem(self, rhs: UVec3) -> UVec3 {
        (*self).rem(rhs)
    }
}

#[cfg(not(target_arch = "spirv"))]
impl AsRef<[u32; 3]> for UVec3 {
    #[inline]
    fn as_ref(&self) -> &[u32; 3] {
        unsafe { &*(self as *const UVec3 as *const [u32; 3]) }
    }
}

#[cfg(not(target_arch = "spirv"))]
impl AsMut<[u32; 3]> for UVec3 {
    #[inline]
    fn as_mut(&mut self) -> &mut [u32; 3] {
        unsafe { &mut *(self as *mut UVec3 as *mut [u32; 3]) }
    }
}

impl Sum for UVec3 {
    #[inline]
    fn sum<I>(iter: I) -> Self
    where
        I: Iterator<Item = Self>,
    {
        iter.fold(Self::ZERO, Self::add)
    }
}

impl<'a> Sum<&'a Self> for UVec3 {
    #[inline]
    fn sum<I>(iter: I) -> Self
    where
        I: Iterator<Item = &'a Self>,
    {
        iter.fold(Self::ZERO, |a, &b| Self::add(a, b))
    }
}

impl Product for UVec3 {
    #[inline]
    fn product<I>(iter: I) -> Self
    where
        I: Iterator<Item = Self>,
    {
        iter.fold(Self::ONE, Self::mul)
    }
}

impl<'a> Product<&'a Self> for UVec3 {
    #[inline]
    fn product<I>(iter: I) -> Self
    where
        I: Iterator<Item = &'a Self>,
    {
        iter.fold(Self::ONE, |a, &b| Self::mul(a, b))
    }
}

impl Not for UVec3 {
    type Output = Self;
    #[inline]
    fn not(self) -> Self::Output {
        Self {
            x: self.x.not(),
            y: self.y.not(),
            z: self.z.not(),
        }
    }
}

impl BitAnd for UVec3 {
    type Output = Self;
    #[inline]
    fn bitand(self, rhs: Self) -> Self::Output {
        Self {
            x: self.x.bitand(rhs.x),
            y: self.y.bitand(rhs.y),
            z: self.z.bitand(rhs.z),
        }
    }
}

impl BitOr for UVec3 {
    type Output = Self;
    #[inline]
    fn bitor(self, rhs: Self) -> Self::Output {
        Self {
            x: self.x.bitor(rhs.x),
            y: self.y.bitor(rhs.y),
            z: self.z.bitor(rhs.z),
        }
    }
}

impl BitXor for UVec3 {
    type Output = Self;
    #[inline]
    fn bitxor(self, rhs: Self) -> Self::Output {
        Self {
            x: self.x.bitxor(rhs.x),
            y: self.y.bitxor(rhs.y),
            z: self.z.bitxor(rhs.z),
        }
    }
}

impl BitAnd<u32> for UVec3 {
    type Output = Self;
    #[inline]
    fn bitand(self, rhs: u32) -> Self::Output {
        Self {
            x: self.x.bitand(rhs),
            y: self.y.bitand(rhs),
            z: self.z.bitand(rhs),
        }
    }
}

impl BitOr<u32> for UVec3 {
    type Output = Self;
    #[inline]
    fn bitor(self, rhs: u32) -> Self::Output {
        Self {
            x: self.x.bitor(rhs),
            y: self.y.bitor(rhs),
            z: self.z.bitor(rhs),
        }
    }
}

impl BitXor<u32> for UVec3 {
    type Output = Self;
    #[inline]
    fn bitxor(self, rhs: u32) -> Self::Output {
        Self {
            x: self.x.bitxor(rhs),
            y: self.y.bitxor(rhs),
            z: self.z.bitxor(rhs),
        }
    }
}

impl Shl<i8> for UVec3 {
    type Output = Self;
    #[inline]
    fn shl(self, rhs: i8) -> Self::Output {
        Self {
            x: self.x.shl(rhs),
            y: self.y.shl(rhs),
            z: self.z.shl(rhs),
        }
    }
}

impl Shr<i8> for UVec3 {
    type Output = Self;
    #[inline]
    fn shr(self, rhs: i8) -> Self::Output {
        Self {
            x: self.x.shr(rhs),
            y: self.y.shr(rhs),
            z: self.z.shr(rhs),
        }
    }
}

impl Shl<i16> for UVec3 {
    type Output = Self;
    #[inline]
    fn shl(self, rhs: i16) -> Self::Output {
        Self {
            x: self.x.shl(rhs),
            y: self.y.shl(rhs),
            z: self.z.shl(rhs),
        }
    }
}

impl Shr<i16> for UVec3 {
    type Output = Self;
    #[inline]
    fn shr(self, rhs: i16) -> Self::Output {
        Self {
            x: self.x.shr(rhs),
            y: self.y.shr(rhs),
            z: self.z.shr(rhs),
        }
    }
}

impl Shl<i32> for UVec3 {
    type Output = Self;
    #[inline]
    fn shl(self, rhs: i32) -> Self::Output {
        Self {
            x: self.x.shl(rhs),
            y: self.y.shl(rhs),
            z: self.z.shl(rhs),
        }
    }
}

impl Shr<i32> for UVec3 {
    type Output = Self;
    #[inline]
    fn shr(self, rhs: i32) -> Self::Output {
        Self {
            x: self.x.shr(rhs),
            y: self.y.shr(rhs),
            z: self.z.shr(rhs),
        }
    }
}

impl Shl<i64> for UVec3 {
    type Output = Self;
    #[inline]
    fn shl(self, rhs: i64) -> Self::Output {
        Self {
            x: self.x.shl(rhs),
            y: self.y.shl(rhs),
            z: self.z.shl(rhs),
        }
    }
}

impl Shr<i64> for UVec3 {
    type Output = Self;
    #[inline]
    fn shr(self, rhs: i64) -> Self::Output {
        Self {
            x: self.x.shr(rhs),
            y: self.y.shr(rhs),
            z: self.z.shr(rhs),
        }
    }
}

impl Shl<u8> for UVec3 {
    type Output = Self;
    #[inline]
    fn shl(self, rhs: u8) -> Self::Output {
        Self {
            x: self.x.shl(rhs),
            y: self.y.shl(rhs),
            z: self.z.shl(rhs),
        }
    }
}

impl Shr<u8> for UVec3 {
    type Output = Self;
    #[inline]
    fn shr(self, rhs: u8) -> Self::Output {
        Self {
            x: self.x.shr(rhs),
            y: self.y.shr(rhs),
            z: self.z.shr(rhs),
        }
    }
}

impl Shl<u16> for UVec3 {
    type Output = Self;
    #[inline]
    fn shl(self, rhs: u16) -> Self::Output {
        Self {
            x: self.x.shl(rhs),
            y: self.y.shl(rhs),
            z: self.z.shl(rhs),
        }
    }
}

impl Shr<u16> for UVec3 {
    type Output = Self;
    #[inline]
    fn shr(self, rhs: u16) -> Self::Output {
        Self {
            x: self.x.shr(rhs),
            y: self.y.shr(rhs),
            z: self.z.shr(rhs),
        }
    }
}

impl Shl<u32> for UVec3 {
    type Output = Self;
    #[inline]
    fn shl(self, rhs: u32) -> Self::Output {
        Self {
            x: self.x.shl(rhs),
            y: self.y.shl(rhs),
            z: self.z.shl(rhs),
        }
    }
}

impl Shr<u32> for UVec3 {
    type Output = Self;
    #[inline]
    fn shr(self, rhs: u32) -> Self::Output {
        Self {
            x: self.x.shr(rhs),
            y: self.y.shr(rhs),
            z: self.z.shr(rhs),
        }
    }
}

impl Shl<u64> for UVec3 {
    type Output = Self;
    #[inline]
    fn shl(self, rhs: u64) -> Self::Output {
        Self {
            x: self.x.shl(rhs),
            y: self.y.shl(rhs),
            z: self.z.shl(rhs),
        }
    }
}

impl Shr<u64> for UVec3 {
    type Output = Self;
    #[inline]
    fn shr(self, rhs: u64) -> Self::Output {
        Self {
            x: self.x.shr(rhs),
            y: self.y.shr(rhs),
            z: self.z.shr(rhs),
        }
    }
}

impl Shl<crate::IVec3> for UVec3 {
    type Output = Self;
    #[inline]
    fn shl(self, rhs: crate::IVec3) -> Self::Output {
        Self {
            x: self.x.shl(rhs.x),
            y: self.y.shl(rhs.y),
            z: self.z.shl(rhs.z),
        }
    }
}

impl Shr<crate::IVec3> for UVec3 {
    type Output = Self;
    #[inline]
    fn shr(self, rhs: crate::IVec3) -> Self::Output {
        Self {
            x: self.x.shr(rhs.x),
            y: self.y.shr(rhs.y),
            z: self.z.shr(rhs.z),
        }
    }
}

impl Shl<crate::UVec3> for UVec3 {
    type Output = Self;
    #[inline]
    fn shl(self, rhs: crate::UVec3) -> Self::Output {
        Self {
            x: self.x.shl(rhs.x),
            y: self.y.shl(rhs.y),
            z: self.z.shl(rhs.z),
        }
    }
}

impl Shr<crate::UVec3> for UVec3 {
    type Output = Self;
    #[inline]
    fn shr(self, rhs: crate::UVec3) -> Self::Output {
        Self {
            x: self.x.shr(rhs.x),
            y: self.y.shr(rhs.y),
            z: self.z.shr(rhs.z),
        }
    }
}

impl Index<usize> for UVec3 {
    type Output = u32;
    #[inline]
    fn index(&self, index: usize) -> &Self::Output {
        match index {
            0 => &self.x,
            1 => &self.y,
            2 => &self.z,
            _ => panic!("index out of bounds"),
        }
    }
}

impl IndexMut<usize> for UVec3 {
    #[inline]
    fn index_mut(&mut self, index: usize) -> &mut Self::Output {
        match index {
            0 => &mut self.x,
            1 => &mut self.y,
            2 => &mut self.z,
            _ => panic!("index out of bounds"),
        }
    }
}

impl fmt::Display for UVec3 {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        write!(f, "[{}, {}, {}]", self.x, self.y, self.z)
    }
}

impl fmt::Debug for UVec3 {
    fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> fmt::Result {
        fmt.debug_tuple(stringify!(UVec3))
            .field(&self.x)
            .field(&self.y)
            .field(&self.z)
            .finish()
    }
}

impl From<[u32; 3]> for UVec3 {
    #[inline]
    fn from(a: [u32; 3]) -> Self {
        Self::new(a[0], a[1], a[2])
    }
}

impl From<UVec3> for [u32; 3] {
    #[inline]
    fn from(v: UVec3) -> Self {
        [v.x, v.y, v.z]
    }
}

impl From<(u32, u32, u32)> for UVec3 {
    #[inline]
    fn from(t: (u32, u32, u32)) -> Self {
        Self::new(t.0, t.1, t.2)
    }
}

impl From<UVec3> for (u32, u32, u32) {
    #[inline]
    fn from(v: UVec3) -> Self {
        (v.x, v.y, v.z)
    }
}

impl From<(UVec2, u32)> for UVec3 {
    #[inline]
    fn from((v, z): (UVec2, u32)) -> Self {
        Self::new(v.x, v.y, z)
    }
}

impl From<U8Vec3> for UVec3 {
    #[inline]
    fn from(v: U8Vec3) -> Self {
        Self::new(u32::from(v.x), u32::from(v.y), u32::from(v.z))
    }
}

impl From<U16Vec3> for UVec3 {
    #[inline]
    fn from(v: U16Vec3) -> Self {
        Self::new(u32::from(v.x), u32::from(v.y), u32::from(v.z))
    }
}

impl TryFrom<I8Vec3> for UVec3 {
    type Error = core::num::TryFromIntError;

    #[inline]
    fn try_from(v: I8Vec3) -> Result<Self, Self::Error> {
        Ok(Self::new(
            u32::try_from(v.x)?,
            u32::try_from(v.y)?,
            u32::try_from(v.z)?,
        ))
    }
}

impl TryFrom<I16Vec3> for UVec3 {
    type Error = core::num::TryFromIntError;

    #[inline]
    fn try_from(v: I16Vec3) -> Result<Self, Self::Error> {
        Ok(Self::new(
            u32::try_from(v.x)?,
            u32::try_from(v.y)?,
            u32::try_from(v.z)?,
        ))
    }
}

impl TryFrom<IVec3> for UVec3 {
    type Error = core::num::TryFromIntError;

    #[inline]
    fn try_from(v: IVec3) -> Result<Self, Self::Error> {
        Ok(Self::new(
            u32::try_from(v.x)?,
            u32::try_from(v.y)?,
            u32::try_from(v.z)?,
        ))
    }
}

impl TryFrom<I64Vec3> for UVec3 {
    type Error = core::num::TryFromIntError;

    #[inline]
    fn try_from(v: I64Vec3) -> Result<Self, Self::Error> {
        Ok(Self::new(
            u32::try_from(v.x)?,
            u32::try_from(v.y)?,
            u32::try_from(v.z)?,
        ))
    }
}

impl TryFrom<U64Vec3> for UVec3 {
    type Error = core::num::TryFromIntError;

    #[inline]
    fn try_from(v: U64Vec3) -> Result<Self, Self::Error> {
        Ok(Self::new(
            u32::try_from(v.x)?,
            u32::try_from(v.y)?,
            u32::try_from(v.z)?,
        ))
    }
}

impl From<BVec3> for UVec3 {
    #[inline]
    fn from(v: BVec3) -> Self {
        Self::new(u32::from(v.x), u32::from(v.y), u32::from(v.z))
    }
}

impl From<BVec3A> for UVec3 {
    #[inline]
    fn from(v: BVec3A) -> Self {
        let bool_array: [bool; 3] = v.into();
        Self::new(
            u32::from(bool_array[0]),
            u32::from(bool_array[1]),
            u32::from(bool_array[2]),
        )
    }
}
